%
% Nuweb: A Simple Literate Programming Tool
%


\documentclass{report}
\usepackage{a4wide,nuweb,latexsym,mflogo,newlfont,shortvrb,version}

\MakeShortVerb{\|}

\providecommand{\hidecode}{false}
\ifthenelse{\equal{\hidecode}{true}}%
    {\excludeversion{code}}{\includeversion{code}}

\newcommand{\HTML}{\textsc{HTML}}


\begin{document}
\pagenumbering{roman}

\title{Nuweb \\ A Simple Literate Programming Tool \\ version 0.90}
\date{15th May 1997}
\author{Preston Briggs\thanks{This work has been supported by ARPA,
through ONR grant N00014-91-J-1989.}
\\ \sl preston@@cs.rice.edu
\\ \HTML\ scrap generator by John D. Ramsdell
\\ \sl ramsdell@@mitre.org
\\ Revised by Reuben Thomas
\\ \sl Reuben.Thomas@@cl.cam.ac.uk}
\maketitle

\tableofcontents
\pagenumbering{arabic}



\chapter{Introduction}

In 1984, Knuth introduced the idea of \emph{literate programming} and
described a pair of tools to support the practice~\cite{knuth:84}. His
approach was to combine Pascal code with \TeX\ documentation to produce a new
language, |WEB|, that offered programmers a superior approach to
programming. He wrote several programs in |WEB|, including |weave|
and |tangle|, the programs used to support literate programming. The
idea was that a programmer wrote one document, the web file, that combined
documentation (written in \TeX~\cite{texbook}) with code (written in Pascal).

Running |tangle| on the web file would produce a complete Pascal
program, ready for compilation by an ordinary Pascal compiler. The primary
function of |tangle| is to allow the programmer to present elements of
the program in any desired order, regardless of the restrictions imposed by
the programming language. Thus, the programmer is free to present his program
in a top-down fashion, bottom-up fashion, or whatever seems best in terms of
promoting understanding and maintenance.

Running |weave| on the web file would produce a \TeX\ file, ready to be
processed by \TeX\@@. The resulting document included a variety of
automatically generated indices and cross-references that made it much easier
to navigate the code. Additionally, all of the code sections were
automatically pretty printed, resulting in a quite impressive document.

Knuth also wrote the programs for \TeX\ and \MF\ entirely in |WEB|,
eventually publishing them in book form~\cite{tex:program,metafont:program}.
These are probably the largest programs ever published in a readable form.

Inspired by Knuth's example, many people have experimented with |WEB|\@@.
Some people have even built \mbox{\texttt{WEB}-like} tools for their own
favorite combinations of programming language and typesetting language. For
example, |CWEB|, Knuth's current system of choice, works with a combination
of C (or C++) and \TeX~\cite{levy:90}. Another system, FunnelWeb, is
independent of any programming language and only mildly dependent on
\TeX~\cite{funnelweb}. Inspired by the versatility of FunnelWeb and by the
daunting size of its documentation, I decided to write my own, very simple,
tool for literate programming.%
\footnote{There is another system similar to mine, written by Norman Ramsey,
called noweb~\cite{noweb}. It perhaps suffers from being overly
Unix-dependent and requiring several programs to use. On the other hand, its
command syntax is very nice. In any case, nuweb certainly owes its name and a
number of features to his inspiration.}



\section{Nuweb}

Nuweb works with any programming language and \LaTeXe~\cite{latex}. (Earlier
versions of nuweb work with \LaTeX2.09.) I wanted to use \LaTeX\ because it
supports a multi-level sectioning scheme and has facilities for drawing
figures. I wanted to be able to work with arbitrary programming languages
because my friends and I write programs in many languages (and sometimes
combinations of several languages), such as C, Fortran, C++, yacc, lex,
Scheme, assembly, Postscript, and so forth. The need to support arbitrary
programming languages has many consequences:
\begin{description}
\item[No pretty printing] Both |WEB| and |CWEB| are able to
  pretty print the code sections of their documents because they
  understand the language well enough to parse it. Since we want to use
  \emph{any} language, we've got to abandon this feature.
\item[Manual indexing of identifiers] Because |WEB| knows about Pascal,
  it is able to construct an index of all the identifiers occurring in
  the code sections (filtering out keywords and the standard type
  identifiers). Unfortunately, this isn't as easy in our case. We don't
  know what an identifier looks like in each language and we certainly
  don't know all the keywords. (On the other hand, see the end of
  section~\ref{writingnuweb}.)
\end{description}
Of course, we've got to have some compensation for our losses or the whole
idea would be a waste. Here are the advantages I can see:
\begin{description}
\item[Simplicity] The majority of the commands in |WEB| are
  concerned with control of the automatic pretty printing. Since we
  don't pretty print, many commands are eliminated. A further set of
  commands is subsumed by \LaTeX\ and may also be eliminated. As a
  result, our set of commands is reduced to only four members. This
  simplicity is also reflected in the size of this tool, which is quite a bit
  smaller than the tools used with other approaches.
\item[No pretty printing] Everyone disagrees about how their code
  should look, so automatic formatting annoys many people. One approach
  is to provide ways to control the formatting. Our approach is
  simpler---we perform no automatic formatting and therefore allow the
  programmer complete control of code layout.
\item[Control] We also offer the programmer complete control of the
  layout of his program files (the files generated during tangling). Of
  course, this is essential for languages that are sensitive to layout;
  but it is also important in many practical situations, such as
  debugging.
\item[Speed] Since nuweb doesn't do too much, the nuweb tool runs
  quickly. I combine the functions of |tangle| and |weave| into
  a single program that performs both functions at once.
\item[Page numbers] Inspired by the example of noweb, nuweb refers to
  all scraps by page number to simplify navigation. If there are
  multiple scraps on a page (say page~17), they are distinguished by
  lower-case letters: 17a, 17b, and so forth).
\item[Multiple file output] The programmer may specify more than one
  program file in a single nuweb file. This is required when constructing
  programs in a combination of languages (say, Fortran and C)\@@. It's also
  an advantage when constructing very large programs that would require
  a lot of compile time.
\end{description}

\label{dummy}
A further reduction in compilation time is achieved by first writing each
program file to a temporary location, then comparing the temporary file with
the old version of the file. If there is no difference, the temporary file
can be deleted. If the files differ, the old version is deleted and the
temporary file renamed. This approach works well in combination with
|make| (or similar tools), since |make| will avoid recompiling
untouched program files. However, this does mean that you must run nuweb
manually whenever you edit your web file (or run it automatically every time
you use |make|). A solution is to produce a dummy program file, say
|dummy|, and add a rule to the makefile like:
\begin{verbatim}
dummy: web.w
        rm dummy
        nuweb web.w
\end{verbatim}
If |dummy| is made a dependent of |.INIT| or some other target that
is made every time that |make| is run, then the web file will
automatically be reprocessed whenever it is changed. The command
|rm dummy| is needed to make sure that the file is rewritten every time
the web is processed.

If you take a more modular approach, and only generate one program file from
each web file, it is better to introduce an implicit rule for making program
files from web files such as:
\begin{verbatim}
.w.c:
	nuweb -tc $<
\end{verbatim}
Note the use of the the |-t| flag to stop the documentation file being
generated, and the |-c| flag (there's no point comparing the new version
of the C file with the old, as we already know it needs to be remade).

\subsection{Nuweb and \HTML}

In addition to producing \LaTeX\ source, nuweb can be used to generate
HyperText Markup Language (\HTML), the markup language used by the World Wide
Web.  This can be done either from \LaTeX\ or \HTML\ source. With a \LaTeX\
source, the tools which generate \HTML\ automatically produce hypertext links.
The remaining text describes the use of \LaTeX\ markup; the use of \HTML\
markup follows the same pattern.



\section{Writing Nuweb} \label{writingnuweb}

The bulk of a nuweb file will be ordinary \LaTeX\@@. In fact, any \LaTeX\
file can serve as input to nuweb and will be simply copied through unchanged
to the documentation file---unless a nuweb command is discovered. All nuweb
commands begin with an at sign (|@@|).  Therefore, a file without
at-signs will be copied unchanged.  Nuweb commands are used to specify
\emph{program files}, define \emph{macros}, and delimit \emph{scraps}. These
are the only features of interest to the nuweb tool---all else is simply text
to be copied to the documentation file.


\subsection{The Major Commands}

Files and macros are defined with the following commands:
\begin{description}
\item[\texttt{@@o} \textit{file-name flags scrap}] Output a program file. The
  file name is terminated by whitespace.
\item[\texttt{@@d} \textit{macro-name scrap}] Define a macro. The macro name
  is terminated by a return or the beginning of a scrap.
\end{description}
A specific file may be specified several times, with the definitions being
written out, one after another, in the order they appear. The definitions
of macros may be similarly divided.

Macro names may be (almost) any well-formed \TeX\ string. Changing fonts is
allowed, as is using math mode; however, since in the |alltt| environment
used to display code the \$ sign is an ordinary character, the commands |\(|
and |\)| should be used instead to enter and leave math mode. Also, spaces
are significant, so avoid constructs such as |{\tt xyz}|; use |\texttt{xyz}|
instead. |\verb| cannot be used in macro names; |\tt| or |\texttt| should be
used instead. When producing \HTML, macros are displayed in a preformatted
element (|PRE|), so macros may contain one or more |A|, |B|, |I|, |U|, or |P|
elements or data characters.

Very long scraps may be allowed to break across a page if declared with
|@@O| or |@@D| (instead of |@@o| and |@@d|). This doesn't
work very well as a default, since far too many short scraps will be broken
across pages; however, as a user-controlled option, it seems very useful.  No
distinction is made between the upper case and lower case forms of these
commands when generating \HTML\@@.

\subsubsection{Scraps}

Scraps have specific start and end markers to allow precise control
over the contents and layout. Note that any amount of whitespace (including
carriage returns) may appear between a name and the beginning of a scrap. A
scrap has the form:
\begin{description}
\item[\texttt{@@\{}\textit{anything}\texttt{@@\}}] where the scrap body
  includes every character in \textit{anything}---all the blanks, all the
  tabs, all the carriage returns.
\end{description}
Inside a scrap, we may invoke a macro:
\begin{description}
\item[\texttt{@@<}\textit{macro-name}\texttt{@@>}] Causes the macro
  \textit{macro-name} to be expanded inline as the code is written out
  to a file. Recursive macro invocations are not allowed.
\end{description}
Note that macro names may be abbreviated, either during invocation or
definition. For example, it would be tedious to have to type
the macro name
\begin{quote}
|@@d Check for terminating at-sequence and return name if found|
\end{quote}
repeatedly. Therefore, we provide a mechanism (borrowed from Knuth) of
indicating abbreviated names.
\begin{quote}
|@@d Check for terminating...|
\end{quote}
The programmer need only type enough characters to uniquely identify the
macro name, followed by three periods. An abbreviation may even occur before
the full version; nuweb simply preserves the longest version of a macro name.
Note also that strings of blanks and tabs are replaced by a single blank.

When scraps are written to a program file or documentation file, tabs are
expanded into spaces by default. Currently, I assume tab stops are set every
eight characters. Furthermore, when a macro is expanded in a scrap, the body
of the macro is indented to match the indentation of the macro invocation.
Therefore, care must be taken with languages such as Fortran that are
sensitive to indentation. These default behaviors may be changed for each
program file (see below).

\subsubsection{Flags}

When defining a program file, various flags may be used to control its
appearance. The flags are intended to make life a little easier for
programmers using certain languages. They introduce little language
dependencies; however, they do so only for a particular file. Thus it is still
easy to mix languages within a single document. There are three per-file
flags:
\begin{description}
\item[\tt -d] Forces the creation of |#line| directives in the
  program file. These are useful with C (and sometimes C++ and Fortran) on
  many systems since they cause the compiler's error messages to
  refer to the web file rather than the program file. Similarly, they
  allow source debugging in terms of the web file.
\item[\tt -i] Suppresses the indentation of macros. That is, when a
  macro is expanded in a scrap, it will \emph{not} be indented to
  match the indentation of the macro invocation. This flag seems
  most useful for Fortran programmers.
\item[\tt -t] Suppresses expansion of tabs in the program file. This
  feature is useful when generating |make| files.
\end{description}


\subsection{The Minor Commands}

There are two very low-level utility commands that may appear anywhere in the
web file.
\begin{description}
\item[\tt @@@@] Causes a single at sign to be copied into the program.
\item[\texttt{@@i} \textit{file-name}] Includes a file. Includes may be
  nested, though there is currently a limit of 16~levels. The file name
  should be complete (no extension will be appended) and should be
  terminated by a newline.
\end{description}


\subsection{Indices} \label{makeindices}

Finally, there are three commands used to create indices to the macro names,
file definitions, and user-specified identifiers.
\begin{description}
\item[\tt @@f] Create an index of file names.
\item[\tt @@m] Create an index of macro names.
\item[\tt @@u] Create an index of user-specified identifiers.
\end{description}
These commands produce a series of |\item| commands in the \LaTeX\ file,
and so must be placed within a list environment of some sort. If there is
only one index, then the |theindex| environment is suitable; otherwise,
the |nuweb| package (see section~\ref{package}) supplies the
|nuwebindex| environment. A typical index will look like this:
\begin{verbatim}
\begin{nuwebindex}[2]{\section{Index}}
@@u
\end{nuwebindex}
\end{verbatim}
The optional argument to the |nuwebindex| environment gives the number of
columns to use, which must be from one to ten. The default is two, which is
usually appropriate, but one is useful for the macros index. The second
argument is text which will be put in a single column just before the index,
and kept on the same page. This prevents a problem where having a |\section|
which starts at the top of a page can cause a page break before the
|nuwebindex| environment, because when the |\begin{nuwebindex}| is executed
the |\section| has not yet been pushed to the top of the following page, so a
page break is inserted after it, even though later it will turn out to be
unnecessary.

Index entries are not broken over a column or page; this prevents problems as
nuweb generates long index lists for frequently used identifiers, which can
span several lines. Unlike the |theindex| environment, no new chapter is
started; this can be done manually if desired, using a command such as
|\chapter*{Index}|.

Identifiers must be explicitly specified for inclusion in the |@@u|
index. By convention, each identifier is marked at the point of its
definition; all references to each identifier inside scraps will be
discovered automatically. To mark an identifier for inclusion in the index,
it is included after the sign \verb!@@|! at the end of a scrap. For example,
\begin{quote}
\begin{verbatim}
@@d a scrap
@@{ Let's pretend we're declaring the variables
FOO and BAR inside this scrap.
@@| FOO BAR
@@}
\end{verbatim}
\end{quote}
I've used alphabetic identifiers in this example, but any string of
characters (not including white\-space or |@@| characters) will do.
Therefore, it's possible to add index entries for things like |<<=| if
desired. An identifier may be declared in more than one scrap.

In the generated index, each identifier appears with a list of all the scraps
using and defining it, where the defining scraps are distinguished by
underlining. If the |-n| flag is used to nuweb (see
section~\ref{running}), these scrap numbers will not be page numbers. Note
that the identifier doesn't actually have to appear in the defining scrap; it
just has to be in the list of definitions at the end of a scrap.


\subsection{The \textsf{nuweb} package} \label{package}

When nuweb is used to produce \LaTeX\ output, the \textsf{nuweb} package must
be used, as it provides commands that nuweb uses to perform its formatting.
If you want to change the way scraps are formatted, you can redefine these
commands; for details, see chapter~\ifthenelse{\equal{\hidecode}{true}}{4 of
the full documentation (produced from
\texttt{nuweb.tex}).}{\ref{formattingscraps}.}

Two facilities are provided for the user: the \texttt{nuwebindex} environment
(see section~\ref{makeindices}), and commands to add a terminating symbol to
the end of each scrap; some authors feel this makes webs easier to read. The
commands are:
\begin{quote}
|\nuwebterminator{|\textit{command}|}|
\end{quote}
where \textit{command} is executed at the end of every scrap. In earlier
versions of nuweb, a diamond ($\Diamond$) was used; to obtain this effect,
use the command
\begin{quote}
|\nuwebterminator{\(\Diamond\)}|
\end{quote}
To have no terminating symbol (the default), use the command
\begin{quote}
|\nuwebnoterminator|
\end{quote}



\section{Running Nuweb} \label{running}

Nuweb is invoked using the following command:
\begin{quote}
\texttt{nuweb} \textit{flags [\texttt{-o} file] file}\dots
\end{quote}
One or more files may be processed at a time; each file may have the name of
its documentation file specified, using \mbox{\texttt{-o} \textit{file}}. The
documentation file name must include the extension. If a file name has no
extension, |.w| will be appended.  \LaTeX\ suitable for translation into
\HTML\ by {\LaTeX}2\HTML\ will be produced from files whose name ends with
|.hw|; source marked up in \HTML\ should have the extension |.htw|.
While a file name may specify a file in another directory, the resulting
documentation file will always be created in the current directory unless the
output filename is specified using the |-o| flag. For example,
\begin{quote}
\texttt{nuweb /foo/bar/quux}
\end{quote}
will take as input the file |/foo/bar/quux.w| and will create the file
|quux.tex| in the current directory.

By default, nuweb performs both tangling and weaving at the same time.
Normally, this is not a bottleneck in the compilation process; however, it's
possible to achieve slightly faster throughput by avoiding one or another of
the default functions using command-line flags. There are currently five
possible flags:
\begin{description}
\item[\texttt{-o} \textit{file}] Call the documentation file \textit{file}.
The filename must include the extension.
\item[\tt -t] Suppress generation of the documentation file.
\item[\tt -p] Suppress generation of the program files.
\item[\tt -c] Avoid testing program files for change before updating them.
\item[\tt -C] Ignore |#line| directives when testing program files for
change. This avoids source files being regenerated just because you have
changed the line numbers at which their code appears in the nuweb document.
Using this option will save a lot of remaking, but may result in debug
information being out of date. Compiler error messages should always be
correct, though.
\end{description}
Thus, the command
\begin{quote} |nuweb -to /foo/bar/quux| \end{quote}
would simply scan the input and produce no program at all.

There are three additional command-line flags:
\begin{description}
\item[\tt -v] For ``verbose'', causes nuweb to write information about
  its progress to |stderr|.
\item[\tt -n] Forces scraps to be numbered sequentially from~1
  (instead of using page numbers). This form is perhaps more desirable
  for small webs. This option applies only when scraps are generated in
  \LaTeX\ format.
\label{skipoption}
\item[\tt -s] Normally, if |@@{| is followed by a newline, the newline
  is ignored when formatting the scrap, and similarly with a newline before
  |@@}| or \verb!@@|!. This allows source code to start on the line
  after |@@{| and end on the line before |@@}| or \verb!@@|!, which
  is convenient for editing, without leaving blank lines in the \LaTeX\ file.
  Using this flag, the scraps are formatted verbatim; scraps are always
  written out to program files verbatim.
\end{description}
It is worth noting that source code often contains wide lines, so when
generating \LaTeX\ documents it may help to use a package such as
\textsf{a4wide} to avoid getting too many ``\texttt{overfull hbox}''
messages.



\section{Generating \HTML}

Nikos Drakos's {\LaTeX}2\HTML~\cite{drakos:94} can be used to translate
\LaTeX\ with embedded \HTML\ scraps into \HTML\@@.  Be sure to use the package
\textsf{html} so that \LaTeX\ will understand the hypertext commands.  When
translating into \HTML, do not allow a document to be split by specifying
``|-split 0|''. You need not generate navigation links, so also specify
``|-no_navigation|''.

While preparing a web, you may want to view the program's scraps without
taking the time to run {\LaTeX}2\HTML\@@.  Simply rename the generated
|.tex| file so that its file name ends with |.html|, and view that
file.  The documentation section will be jumbled, but the scraps will be
clear.



\section{Restrictions}

Because nuweb is intended to be a simple tool, I've established a few
restrictions. Over time, some of these may be eliminated; others seem
fundamental.

\begin{itemize}
\item File names and indexed identifiers must not contain any |@@|
  signs. This may be changed in future.
\item Anything is allowed in the body of a scrap; however, very
  large scraps (horizontally or vertically) do not typeset well.
\item Temporary files (created for comparison with the existing
  program files) are placed in the current directory. Since they may be
  renamed to an program file name, all the program files must be on the
  same file system as the current directory.
\item Because page numbers cannot be determined until the document has
  been typeset, we have to rerun nuweb after \LaTeX\ to obtain a clean
  version of the document (very similar to the way we sometimes have
  to rerun \LaTeX\ to obtain an up-to-date table of contents after
  significant edits).  Nuweb will warn (in most cases) when this needs
  to be done; in the remaining cases, \LaTeX\ will warn that labels
  may have changed.
\item Tabs are expanded to eight spaces; a per-file flag might in
  future be added to specify how many spaces should replace a tab.
\item Nuweb is still not completely portable: it expects file-names to use
  |/| to delimit directories.
\end{itemize}



\section{Acknowledgements}

Several people have contributed their times, ideas, and debugging skills. In
particular, I'd like to acknowledge the contributions of Osman Buyukisik,
Manuel Carriba, Adrian Clarke, Tim Harvey, Michael Lewis, Walter Ravenek, Rob
Shillingsburg, Kayvan Sylvan, Dominique de~Waleffe, and Scott Warren.  Of
course, most of these people would never have heard of nuweb (or many other
tools) without the efforts of George Greenwade. \\[0.3ex]
\hspace*{\fill}\textbf{Preston Briggs}

\vspace{0.5cm}

\noindent Preston Briggs and John Ramsdell encouraged and advised me when I
suggested my revisions, and kindly permitted me to distribute the result.
\\[0.3ex] \hspace*{\fill}\textbf{Reuben Thomas}



\begin{code}
\chapter{The Main Program}

Processing a web requires three major steps:

\begin{enumerate}
\item Read the source, accumulating file names, macro names, scraps,
and lists of cross-references.
\item Re-read the source, copying out to the documentation file, formatting
and cross-referencing the scraps.
\item Traverse the list of files names. For each file name:
    \begin{enumerate}
    \item Dump all the defining scraps into a temporary file.
    \item If the file already exists and is unchanged, delete the
    temporary file; otherwise, rename the temporary file.
    \end{enumerate}
\end{enumerate}



\section{Files}

I have divided the program into several files for quicker
recompilation during development.

The dummy file |dummy| is used to make the makefile work (see
section~\ref{dummy}).

@o dummy
@{@}

@o global.h -d
@{
@<Include files@>
@<Type declarations@>
@<Global variable declarations@>
@<Function prototypes@>
@}

We'll need some of the standard system include files.

@d Include files
@{
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <limits.h>
@| FILE stderr exit fprintf fputs fopen fclose getc putc strlen
toupper isupper islower isgraph isspace tmpnam remove malloc size_t CHAR_MAX
@}

I also like to use |TRUE| and |FALSE| in my code.
I'd use an |enum| here, except that some systems seem to provide
definitions of |TRUE| and |FALSE| by default.  The following
code seems to work on all the local systems.

@d Type dec...
@{
#ifndef FALSE
#define FALSE 0
#endif
#ifndef TRUE
#define TRUE 1
#endif
@| FALSE TRUE @}


\subsection{The Main Files}

The code is divided into five main files (introduced here) and five support
files (introduced in the next section).

The file |main.c| contains the driver for the whole program (see
section~\ref{main-routine}).

@o main.c -d
@{#include "global.h"
@}

The first pass over the source file is contained in |pass1.c|.
It handles collection of all the file names, macros names, and scraps
(see section~\ref{pass-one}).

@o pass1.c -d
@{#include "global.h"
@}

The |.tex| file is created during a second pass over the source file.
The file |latex.c| contains the code controlling the construction of the
|.tex| file (see section~\ref{latex-file}).

@o latex.c -d
@{#include "global.h"
@}

The file |html.c| contains the code controlling the construction of the
|.tex| file appropriate for use with {\LaTeX}2\HTML, or the |.html|
file (see section~\ref{html-file}).

@o html.c -d
@{#include "global.h"
@}

The code controlling the creation of the program files is in |program.c|
(see section~\ref{program-files}).

@o program.c -d
@{#include "global.h"
@}


\subsection{Support Files}

The support files contain a variety of support routines used to define and
manipulate the major data abstractions. The file |input.c| holds all the
routines used for referring to source files (see section~\ref{source-files}).

@o input.c -d
@{#include "global.h"
@}

Creation and lookup of scraps is handled by routines in |scraps.c|
(see section~\ref{scraps}).

@o scraps.c -d
@{#include "global.h"
@}


The handling of file names and macro names is detailed in |names.c|
(see section~\ref{names}).

@o names.c -d
@{#include "global.h"
@}

Memory allocation and deallocation is handled by routines in |arena.c|
(see Section~\ref{memory-management}).
@o arena.c
@{#include "global.h"
@}

Finally, I need a file containing definitions of all the global variables.
@o global.c
@{#include "global.h"
@<Global variable definitions@>
@}



\section{The Main Routine} \label{main-routine}

The main routine is quite simple in structure. It wades through the optional
command-line arguments, then handles any files listed on the command line.

@o main.c -d
@{
int main(int argc, char *argv[])
{
  int arg = 1;
  @<Interpret command-line arguments@>
  @<Process the remaining arguments (file names)@>
  exit(0);
}
@| main @}


\subsection{Command-Line Arguments}

There are eight possible command-line flags:
\begin{description}
\item[\tt -o] Give a name for the documentation file.
\item[\tt -t] Suppresses generation of the |.tex| file.
\item[\tt -p] Suppresses generation of the program files.
\item[\tt -c] Forces program files to overwrite old files of the same
  name without comparing for equality first.
\item[\tt -C] Ignores |#line| directives when comparing files.
\item[\tt -s] Skips a newline after |@@{| and before |@@}| or
\verb!@@|!.
\item[\tt -v] The verbose flag. Forces output of progress reports.
\item[\tt -n] Forces sequential numbering of scraps (instead of page
  numbers).
\end{description}

\noindent
Global flags are declared for each of the arguments.

@d Global variable dec...
@{
extern int tex_flag;      /* if FALSE, don't emit the documentation file */
extern int html_flag;     /* if TRUE, emit HTML scraps. */
extern int hw_flag;       /* if TRUE, emit HTML scraps from LaTeX input. */
extern int program_flag;  /* if FALSE, don't emit the program files */
extern int compare_flag;  /* if FALSE, overwrite without comparison */
extern int compline_flag; /* if TRUE, ignore #line directives while comparing */
extern int verbose_flag;  /* if TRUE, write progress information */
extern int number_flag;   /* if TRUE, use a sequential numbering scheme */
extern int skipnl_flag;   /* if TRUE, skip a newline after @@{ and before @@} */
@| tex_flag html_flag hw_flag program_flag compare_flag compline_flag verbose_flag
number_flag skipnl_flag @}

The flags are all initialized for correct default behavior.

@d Global variable def...
@{
int tex_flag = TRUE;
int html_flag = FALSE;
int hw_flag = FALSE;
int program_flag = TRUE;
int compare_flag = TRUE;
int compline_flag = FALSE;
int verbose_flag = FALSE;
int number_flag = FALSE;
int skipnl_flag = TRUE;
@}


We save the invocation name of the command in a global variable
|command_name| for use in error messages.

@d Global variable dec...
@{extern char *command_name;
@| command_name @}

@d Global variable def...
@{char *command_name = NULL;
@}

The invocation name is conventionally passed in |argv[0]|.

@d Interpret com...
@{command_name = argv[0];
@}

We have a standard error message that we print when the command-line syntax
is violated. Afterwards, we always exit.

@d Show usage and exit
@{
fprintf(stderr, "Usage is: %s [-cCnpstv] [[-o file] file]...\n", command_name);
exit(-1);
@}

We need to examine the remaining entries in |argv|, looking for
command-line arguments.

@d Interpret com...
@{
while (arg < argc) {
  char *s = argv[arg];
  if (*s++ == '-' && *s != 'o') {
    @<Interpret the argument string {\tt s}@>
    arg++;
  }
  else break;
}
@}

Several flags can be stacked behind a single minus sign; therefore,
we've got to loop through the string, handling them all.

@d Interpret the...
@{
{
  char c = *s++;
  while (c) {
    switch (c) {
      case 'c': compare_flag = FALSE;
		break;
      case 'C': compline_flag = TRUE;
		break;
      case 'n': number_flag = TRUE;
		break;
      case 'p': program_flag = FALSE;
		break;
      case 's': skipnl_flag = FALSE;
		break;
      case 't': tex_flag = FALSE;
		break;
      case 'v': verbose_flag = TRUE;
		break;
      default:  fprintf(stderr, "%s: unexpected argument.  ",
			command_name);
      @<Show usage and exit@>
		break;
    }
    c = *s++;
  }
}
@}


\subsection{File Names}

We expect at least one file name. While a missing file name might be
ignored without causing any problems, we take the opportunity to report
the usage convention.

@d Process the remaining...
@{
{
  if (arg >= argc) {
    fprintf(stderr, "%s: expected a file name.  ", command_name);
    @<Show usage and exit@>
  }
  do {
    @<Handle the file name in {\tt argv[arg]}@>
    arg++;
  } while (arg < argc);
}
@}

The code to handle a particular file name is rather more tedious than the
actual processing of the file. A file name may be an arbitrarily complicated
path name, with an optional extension. If no extension is present, we add
|.w| as a default. The extended path name will be kept in a local
variable |source_name|. The resulting documentation file will be written
in the current directory, unless its name has been specified using the
|-o| flag; its name will be kept in the variable |tex_name|.

@d Handle the file...
@{
{
  char source_name[256];
  char tex_name[256];
  char aux_name[256];
  int output_given = FALSE;

  if (strcmp(argv[arg], "-o") == 0) {
      arg++;
      output_given = TRUE;
      strcpy(tex_name, argv[arg++]);
  }

  @<Build {\tt source\_name} and {\tt tex\_name}@>
  @<Process a file@>
}
@}

The function |trim_name| performs the trimming. I bump through all the
characters in |from|, copying all the characters into |to|. At each
slash, I update |trim| to point just past the slash in |from|. The
effect is that |trim| will point at the file name without any leading
directory specifications. The function returns a pointer to the extension.

@d Function pro...
@{char *trim_name(char *from, char *to, char **dot);
@}

@o main.c -d
@{
char *trim_name(char *from, char *to, char **dot)
{
  char *trim = to;
  char c = *from++;

  *dot = NULL;
  while (c) {
    *to++ = c;
    if (c == '/') {
      trim = to;
      *dot = NULL;
    }
    else if (c == '.')
      *dot = to - 1;
    c = *from++;
  }
  *to = '\0';

  return trim;
}
@| trim_name @}

The pointer |dot| is made to point at the file name extension, if
present. If there is no extension, we add |.w| to the source name. If
the output file is not specified, we create the |tex_name| from
|trim|, taking care to get the correct extension; we also create the
|aux_name|. The |html_flag| and |hw_flag| are set in this
scrap.

@d Build {\tt sou...
@{
{
  char *dot;
  char *trim = trim_name(argv[arg], source_name, &dot);

  if (dot) {
    *dot = '\0'; /* produce HTML when the file extension is ".hw" */
    hw_flag = dot[1] == 'h' && dot[2] == 'w' && dot[3] == '\0';
    html_flag = hw_flag ||
         (dot[1] == 'h' && dot[2] == 't' && dot[3] == 'w' && dot[4] == '\0');
    if (!output_given) {
      sprintf(tex_name, (html_flag && !hw_flag) ? "%s.html" : "%s.tex", trim);
      sprintf(aux_name, "%s.aux", trim);
    }
    *dot = '.';
  }
  else {
    if (!output_given) {
      sprintf(tex_name, "%s.tex", trim);
      sprintf(aux_name, "%s.aux", trim);
    }
    strcat(source_name, ".w");
  }
  if (output_given && !html_flag && !hw_flag) {
    char temp[256], *dot;

    strcpy(temp, tex_name);
    trim_name(temp, aux_name, &dot);
    if (dot)
      *dot = '\0';
    strcat(aux_name, ".aux");
  }
}
@}

Now that we're finally ready to process a file, it's not really too complex.
We bundle most of the work into four routines |pass1| (see
section~\ref{pass-one}), |write_tex| (see section~\ref{latex-file}),
|write_html| (see section~\ref{html-file}), and |write_files| (see
section~\ref{program-files}). After we're finished with a particular file, we
must remember to release its storage (see section~\ref{memory-management}).
The sequential numbering of scraps is forced when generating \HTML.

@d Process a file
@{
{
  pass1(source_name);
  if (tex_flag) {
    if (html_flag) {
      int saved_number_flag = number_flag;
      number_flag = TRUE;
      collect_numbers(aux_name);
      write_html(source_name, tex_name);
      number_flag = saved_number_flag;
    }
    else {
      collect_numbers(aux_name);
      write_tex(source_name, tex_name);
    }
  }
  if (program_flag)
    write_files(file_names);
}
@}



\section{Pass One} \label{pass-one}

During the first pass, we scan the file, recording the definitions of
each macro and file and accumulating all the scraps.

@d Function pro...
@{void pass1(char *file_name);
@}


The routine |pass1| takes a single argument, the name of the
source file. It opens the file, then initializes the scrap structures
(see section~\ref{scraps}) and the roots of the file-name tree, the
macro-name tree, and the tree of user-specified index entries (see
section~\ref{names}). After completing all the
necessary preparation, we make a pass over the file, filling in all
our data structures. Next, we search all the scraps for references to
the user-specified index entries. Finally, we must reverse all the
cross-reference lists accumulated while scanning the scraps.

@o pass1.c -d
@{
void pass1(char *file_name)
{
  if (verbose_flag)
    fprintf(stderr, "reading %s\n", file_name);
  source_open(file_name);
  init_scraps();
  macro_names = NULL;
  file_names = NULL;
  user_names = NULL;
  @<Scan the source file, looking for at-sequences@>
  if (tex_flag)
    search();
  @<Reverse cross-reference lists@>
}
@| pass1 @}

The only thing we look for in the first pass are the command
sequences. All ordinary text is skipped entirely.

@d Scan the source file, look...
@{
{
  int c = source_get();
  while (c != EOF) {
    if (c == '@@')
      @<Scan at-sequence@>
    c = source_get();
  }
}
@}

Only four of the at-sequences are interesting during the first pass. We skip
past others immediately.

@d Scan at-sequence
@{
{
  c = source_get();
  switch (c) {
    case 'O':
    case 'o': @<Build program file definition@>
	      break;
    case 'D':
    case 'd': @<Build macro definition@>
	      break;
  }
}
@}


\subsection{Accumulating Definitions}

There are three steps required to handle a definition:
\begin{enumerate}
\item Build an entry for the name so we can look it up later.
\item Collect the scrap and save it in the table of scraps.
\item Attach the scrap to the name.
\end{enumerate}
We go through the same steps for both file names and macro names.

@d Build program file definition
@{
{
  Name *name = collect_file_name(); /* returns a pointer to the name entry */
  int scrap = collect_scrap();	    /* returns an index to the scrap */
  @<Add {\tt scrap} to...@>
}
@}

@d Build macro definition
@{
{
  Name *name = collect_macro_name();
  int scrap = collect_scrap();
  @<Add {\tt scrap} to...@>
}
@}


Since a file or macro may be defined by many scraps, we maintain them
in a simple linked list. The list is actually built in reverse order,
with each new definition being added to the head of the list.
@d Add {\tt scrap} to {\tt name}'s definition list
@{
{
  Scrap_Node *def = (Scrap_Node *) arena_getmem(sizeof(Scrap_Node));
  def->scrap = scrap;
  def->next = name->defs;
  name->defs = def;
}
@}


\subsection{Fixing the Cross References}

Since the definition and reference lists for each name are accumulated
in reverse order, we take the time at the end of |pass1| to
reverse them all so they'll be simpler to print out prettily.
The code for |reverse_lists| appears in section~\ref{names}.

@d Reverse cross-reference lists
@{
{
  reverse_lists(file_names);
  reverse_lists(macro_names);
  reverse_lists(user_names);
}
@}



\section{Writing the \LaTeX\ File} \label{latex-file}

The second pass (invoked via a call to |write_tex|) copies most of
the text from the source file straight into a |.tex| file.
Definitions are formatted slightly and cross-reference information is
printed out.

Note that all the formatting is handled in this section. Almost all of the
commands used are defined by the \textsf{nuweb} \LaTeX\ package, so to change
the formatting, the package should be altered rather than the program. The
exception is formatting of reference lists, which is performed by the
program.

On the other hand, if someone wanted to modify nuweb to work with a different
typesetting system, this would be the place to look.

@d Function...
@{void write_tex(char *file_name, char *tex_name);
@}

We need a few local function declarations before we get into the body
of |write_tex|.

@o latex.c -d
@{
static void copy_scrap(FILE *file);		/* formats the body of a scrap */
static int print_scrap_numbers(FILE *tex_file, Scrap_Node *scraps);
     /* formats a list of scrap numbers */
static void format_entry(Name *name, FILE *tex_file, int file_flag);
     /* formats an index entry */
static void format_user_entry(Name *name, FILE *tex_file);
@}

The routine |write_tex| takes two file names as parameters: the
name of the web source file and the name of the |.tex| file.

@o latex.c -d
@{
void write_tex(char *file_name, char *tex_name)
{
  FILE *tex_file = fopen(tex_name, "w");
  if (tex_file) {
    if (verbose_flag)
      fprintf(stderr, "writing %s\n", tex_name);
    source_open(file_name);
    @<Copy {\tt source\_file} into {\tt tex\_file}@>
    fclose(tex_file);
  }
  else
    fprintf(stderr, "%s: can't open %s\n", command_name, tex_name);
}
@| write_tex @}

We make our second (and final) pass through the source web, this time
copying characters straight into the |.tex| file. However, we keep
an eye peeled for |@@|~characters, which signal a command sequence.

@d Copy {\tt source\_file} into {\tt tex\_file}
@{
{
  int scraps = 1;
  int c = source_get();
  while (c != EOF) {
    if (c == '@@')
      @<Interpret at-sequence@>
    else {
      putc(c, tex_file);
      c = source_get();
    }
  }
}
@}

@d Interpret at-sequence
@{
{
  int big_definition = FALSE;
  c = source_get();
  switch (c) {
    case 'O': big_definition = TRUE;
    case 'o': @<Write program file definition@>
	      c = source_get();
	      break;
    case 'D': big_definition = TRUE;
    case 'd': @<Write macro definition@>
	      c = source_get();
	      break;
    case 'f': @<Write index of file names@>
	      break;
    case 'm': @<Write index of macro names@>
	      break;
    case 'u': @<Write index of user-specified names@>
	      break;
    case '@@': putc(c, tex_file);
    default:  c = source_get();
	      break;
  }
}
@}


\subsection{Formatting Definitions}

We go through a fair amount of effort to format a file definition. The
\LaTeX\ for the previous macro definition should look like this (possibly
modulo the scrap references):
{\small
\begin{verbatim}
\begin{nuwebmacro}{Interpret at-sequence}{38}{}{0}{37}{1}
\verb!{!
  int big_definition = FALSE;
  c = source_get();
  switch (c) \verb!{!
    case 'O': big_definition = TRUE;
    case 'o': \nuwebmacroname{Write program file definition}{39}
              c = source_get();
              break;
    case 'D': big_definition = TRUE;
    case 'd': \nuwebmacroname{Write macro definition}{40}
              c = source_get();
              break;
    case 'f': \nuwebmacroname{Write index of file names}{50}
              break;
    case 'm': \nuwebmacroname{Write index of macro names}{51}
              break;
    case 'u': \nuwebmacroname{Write index of user-specified names}{57}
              break;
    case '@@': putc(c, tex_file);
    default:  c = source_get();
              break;
  \verb!}!
\verb!}!\end{nuwebmacro}
\end{verbatim}}

\noindent
Macro and file definitions are formatted nearly identically.
I've factored the common parts out into separate scraps.

@d Write program file definition
@{
{
  Name *name = collect_file_name();
  if (!number_flag) fprintf(tex_file, "\\label{scrap%d}\n", scraps);
  if (!big_definition) fputs("\\begin{nuwebfile}", tex_file);
  else fputs("\\begin{nuwebbigfile}", tex_file);
  fprintf(tex_file, "{%s}{", name->spelling);
  write_single_scrap_ref(tex_file, scraps++);
  putc('}', tex_file);
  @<Write defs@>
  fputs("{}{0}", tex_file);
  putc('\n', tex_file);
  copy_scrap(tex_file);
  if (!big_definition) fputs("\\end{nuwebfile}\n", tex_file);
  else fputs("\\end{nuwebbigfile}\n", tex_file);
}
@}


I don't format a macro name at all specially, figuring the programmer
might want to use effects like italics or bold face in the name.

@d Write macro definition
@{
{
  Name *name = collect_macro_name();
  if (!number_flag) fprintf(tex_file, " \\label{scrap%d}\n", scraps);
  if (!big_definition) fputs("\\begin{nuwebmacro}", tex_file);
  else fputs("\\begin{nuwebbigmacro}", tex_file);
  fprintf(tex_file, "{%s}{", name->spelling);
  write_single_scrap_ref(tex_file, scraps++);
  putc('}', tex_file);
  @<Write defs@>
  @<Write macro refs@>
  putc('\n', tex_file);
  copy_scrap(tex_file);
  if (!big_definition) fputs("\\end{nuwebmacro}\n", tex_file);
  else fputs("\\end{nuwebbigmacro}\n", tex_file);
}
@}


\subsubsection{Formatting Cross References}

@d Write defs
@{
if (name->defs->next) {
  int nos = print_scrap_numbers(tex_file, name->defs);
  fprintf(tex_file, "{%d}", nos);
}
else fputs("{}{0}", tex_file);
@}

@d Write macro refs
@{
{
  if (name->uses) {
    if (name->uses->next) {
      int nos = print_scrap_numbers(tex_file, name->uses);
      fprintf(tex_file, "{%d}", nos);
    }
    else {
      putc('{', tex_file);
      write_single_scrap_ref(tex_file, name->uses->scrap);
      fputs("}{1}", tex_file);
    }
  }
  else {
    fputs("{}{0}", tex_file);
    fprintf(stderr, "%s: <%s> never referenced.\n",
	    command_name, name->spelling);
  }
}
@}


The function |print_scrap_numbers()| returns the number of numbers printed
so that it can be appended as an argument to the scrap typesetting commands.

@o latex.c -d
@{
static int print_scrap_numbers(FILE *tex_file, Scrap_Node *scraps)
{
  int page, nos = 0;
  putc('{', tex_file);
  write_scrap_ref(tex_file, scraps->scrap, TRUE, &page);
  scraps = scraps->next;
  nos++;
  while (scraps) {
    write_scrap_ref(tex_file, scraps->scrap, FALSE, &page);
    scraps = scraps->next;
    nos++;
  }
  putc('}', tex_file);
  return nos;
}
@| print_scrap_numbers @}


\subsubsection{Formatting a Scrap}

@o latex.c -d
@{
static void copy_scrap(FILE *file)
{
  int indent = 0;
  int c = source_get();
  while (1) {
    switch (c) {
      case '@@':  @<Check at-sequence for end-of-scrap@>
		 break;
      case '\t': @<Expand tab into spaces@>
		 break;
      case '\n': c = source_get();
      		 if (!skipnl_flag || c != '@@' ||
          	     (source_next() != '|' && source_next() != '}')) {
          	   putc('\n', file);
        	   indent = 0;
                 }
	 	 continue;
      case '{':
      case '}':
      case '\\': fprintf(file, "\\verb!%c!", c);
      		 break;
      default:   putc(c, file);
		 indent++;
		 break;
    }
    c = source_get();
  }
}
@| copy_scrap @}

We assume that the environment in which the scrap is formatted (|alltt|
by default) treats braces and the backslash as normal in \LaTeX; thus, these
must be escaped out.

@d Expand tab into spaces
@{
{
  int delta = 8 - (indent % 8);
  indent += delta;
  while (delta > 0) {
    putc(' ', file);
    delta--;
  }
}
@}

@d Check at-sequence...
@{
{
  c = source_get();
  switch (c) {
    case '@@': putc('@@', file);
	      break;
    case '|': @<Skip over index entries@>
    case '}': return;
    case '<': @<Format macro name@>
	      break;
  }
}
@}

There's no need to check for errors or other commands here, since they will
have been dealt with during the first pass.

@d Skip over index entries
@{
{
  do {
    do
      c = source_get();
    while (c != '@@');
    c = source_get();
  } while (c != '}');
}
@}


@d Format macro name
@{
{
  Name *name = collect_scrap_name();
  fprintf(file, "\\nuwebmacroname{%s}{", name->spelling);
  if (name->defs)
    @<Write abbreviated definition list@>
  else {
    putc('?', file);
    fprintf(stderr, "%s: scrap never defined <%s>\n",
	    command_name, name->spelling);
  }
  putc('}', file);
}
@}


@d Write abbreviated definition list
@{
{
  Scrap_Node *p = name->defs;
  write_single_scrap_ref(file, p->scrap);
  p = p->next;
  if (p)
    fputs(",\\dots", file);
}
@}


\subsection{Generating the Indices}

@d Write index of file names
@{
{
  if (file_names)
    format_entry(file_names, tex_file, TRUE);
  c = source_get();
}
@}


@d Write index of macro names
@{
{
  if (macro_names)
    format_entry(macro_names, tex_file, FALSE);
  c = source_get();
}
@}


@o latex.c -d
@{
static void format_entry(Name *name, FILE *tex_file, int file_flag)
{
  while (name) {
    format_entry(name->llink, tex_file, file_flag);
    @<Format an index entry@>
    name = name->rlink;
  }
}
@| format_entry @}


@d Format an index entry
@{
{
  fputs("\\item ", tex_file);
  if (file_flag) {
    fprintf(tex_file, "\\verb@@\"%s\"@@, ", name->spelling);
    @<Write file's defining scrap numbers@>
  }
  else {
    fprintf(tex_file, "\\(\\langle\\)%s, ", name->spelling);
    @<Write defining scrap numbers@>
    fputs("\\(\\rangle\\) ", tex_file);
    @<Write referencing scrap numbers@>
  }
  putc('\n', tex_file);
}
@}


@d Write file's defining scrap numbers
@{
{
  Scrap_Node *p = name->defs;
  if (p->next)
    print_scrap_numbers(tex_file, p);
  else
    write_single_scrap_ref(tex_file, p->scrap);
}
@}

@d Write defining scrap numbers
@{
{
  Scrap_Node *p = name->defs;
  if (p) {
    int page;
    write_scrap_ref(tex_file, p->scrap, TRUE, &page);
    p = p->next;
    while (p) {
      write_scrap_ref(tex_file, p->scrap, FALSE, &page);
      p = p->next;
    }
  }
  else
    putc('?', tex_file);
}
@}

@d Write referencing scrap numbers
@{
{
  Scrap_Node *p = name->uses;
  if (p) {
    if (p->next)
      print_scrap_numbers(tex_file, p);
    else
      write_single_scrap_ref(tex_file, p->scrap);
  }
  else
    fputs("\\textbf{not referenced}", tex_file);
}
@}


@d Write index of user-specified names
@{
{
  if (user_names)
    format_user_entry(user_names, tex_file);
  c = source_get();
}
@}


@o latex.c -d
@{
static void format_user_entry(Name *name, FILE *tex_file)
{
  while (name) {
    format_user_entry(name->llink, tex_file);
    @<Format a user index entry@>
    name = name->rlink;
  }
}
@| format_user_entry @}


@d Format a user index entry
@{
{
  Scrap_Node *uses = name->uses;
  if (uses) {
    int page;
    Scrap_Node *defs = name->defs;
    fprintf(tex_file, "\\item \\verb@@%s@@, ", name->spelling);
    if (uses->scrap < defs->scrap) {
      write_scrap_ref(tex_file, uses->scrap, TRUE, &page);
      uses = uses->next;
    }
    else {
      if (defs->scrap == uses->scrap)
        uses = uses->next;
      fputs("\\underline{", tex_file);
      write_single_scrap_ref(tex_file, defs->scrap);
      putc('}', tex_file);
      page = -2;
      defs = defs->next;
    }
    while (uses || defs) {
      if (uses && (!defs || uses->scrap < defs->scrap)) {
        write_scrap_ref(tex_file, uses->scrap, FALSE, &page);
        uses = uses->next;
      }
      else {
        if (uses && defs->scrap == uses->scrap)
	  uses = uses->next;
        fputs(", \\underline{", tex_file);
        write_single_scrap_ref(tex_file, defs->scrap);
        putc('}', tex_file);
        page = -2;
        defs = defs->next;
      }
    }
    fputs("\n", tex_file);
  }
}
@}


\section{Writing the \LaTeX\ File with \HTML\ Scraps or \HTML\ File}
\label{html-file}

The \HTML\ generated is patterned closely upon the \LaTeX\ generated in
the previous section.\footnote{While writing this section, I tried to follow
Preston's style as displayed in section~\ref{latex-file}---J. D. R.}  When a
file name ends in |.hw| or |.htw|, the second pass (invoked via a
call to |write_html|) copies most of the text from the source file
straight into a |.tex| or |.html| file.  Definitions are formatted
slightly and cross-reference information is printed out.

@d Function...
@{void write_html(char *file_name, char *html_name);
@}

We need a few local function declarations before we get into the body
of |write_html|.

@o html.c -d
@{
static void copy_scrap(FILE *file);     /* formats the body of a scrap */
static void display_scrap_ref(FILE *html_file, int num);
     /* formats a scrap reference */
static void display_scrap_numbers(FILE *html_file, Scrap_Node *scraps);
     /* formats a list of scrap numbers */
static void print_scrap_numbers(FILE *html_file, Scrap_Node *scraps);
     /* pluralizes scrap formats list */
static void format_entry(Name *name, FILE *html_file, int file_flag);
     /* formats an index entry */
static void format_user_entry(Name *name, FILE *html_file);
@}


The routine |write_html| takes two file names as parameters: the
name of the web source file and the name of the documentation file.

@o html.c -d
@{
void write_html(char *file_name, char *html_name)
{
  FILE *html_file = fopen(html_name, "w");
  if (html_file) {
    if (verbose_flag)
      fprintf(stderr, "writing %s\n", html_name);
    source_open(file_name);
    @<Copy {\tt source\_file} into {\tt html\_file}@>
    fclose(html_file);
  }
  else
    fprintf(stderr, "%s: can't open %s\n", command_name, html_name);
}
@| write_html @}


We make our second (and final) pass through the source web, this time
copying characters straight into the |.tex| or |.html| file.
However, we keep an eye peeled for |@@|~characters, which signal a
command sequence.

@d Copy {\tt source\_file} into {\tt html\_file}
@{
{
  int scraps = 1;
  int c = source_get();
  while (c != EOF) {
    if (c == '@@')
      @<Interpret \HTML\ at-sequence@>
    else {
      putc(c, html_file);
      c = source_get();
    }
  }
}
@}

@d Interpret \HTML\ at-sequence
@{
{
  c = source_get();
  switch (c) {
    case 'O':
    case 'o': @<Write \HTML\ program file definition@>
	      break;
    case 'D':
    case 'd': @<Write \HTML\ macro definition@>
	      break;
    case 'f': @<Write \HTML\ index of file names@>
	      break;
    case 'm': @<Write \HTML\ index of macro names@>
	      break;
    case 'u': @<Write \HTML\ index of user-specified names@>
	      break;
    case '@@': putc(c, html_file);
    default:  c = source_get();
	      break;
  }
}
@}


\subsection{Formatting Definitions}

We go through only a little amount of effort to format a definition.
The \HTML\ for the previous macro definition should look like this
(perhaps modulo the scrap references):

\begin{verbatim}
<pre>
<a name="nuweb68">&lt;Interpret \HTML\ at-sequence 68&gt;</a> =
{
  c = source_get();
  switch (c) {
    case 'O':
    case 'o': &lt;Write HTML program file definition <a href="#nuweb69">69</a>&gt;
              break;
    case 'D':
    case 'd': &lt;Write HTML macro definition <a href="#nuweb71">71</a>&gt;
              break;
    case 'f': &lt;Write HTML index of file names <a href="#nuweb86">86</a>&gt;
              break;
    case 'm': &lt;Write HTML index of macro names <a href="#nuweb87">87</a>&gt;
              break;
    case 'u': &lt;Write HTML index of user-specified names <a href="#nuweb93">93</a>&gt;
              break;
    case '@@': putc(c, html_file);
    default:  c = source_get();
              break;
  }
}&lt;&gt;</pre>
Macro referenced in scrap <a href="#nuweb67">67</a>.
<br>
\end{verbatim}

Macro and file definitions are formatted nearly identically.
I've factored the common parts out into separate scraps.

@d Write \HTML\ program file definition
@{
{
  Name *name = collect_file_name();
  @<Begin \HTML\ scrap environment@>
  @<Write \HTML\ program file declaration@>
  scraps++;
  @<Fill in the middle of \HTML\ scrap environment@>
  @<Write \HTML\ file defs@>
  @<Finish \HTML\ scrap environment@>
}
@}

@d Write \HTML\ program file declaration
@{
  fputs("<a name=\"nuweb", html_file);
  write_single_scrap_ref(html_file, scraps);
  fprintf(html_file, "\"><code>\"%s\"</code> ", name->spelling);
  write_single_scrap_ref(html_file, scraps);
  fputs("</a> =\n", html_file);
@}

@d Write \HTML\ macro definition
@{
{
  Name *name = collect_macro_name();
  @<Begin \HTML\ scrap environment@>
  @<Write \HTML\ macro declaration@>
  scraps++;
  @<Fill in the middle of \HTML\ scrap environment@>
  @<Write \HTML\ macro defs@>
  @<Write \HTML\ macro refs@>
  @<Finish \HTML\ scrap environment@>
}
@}

I don't format a macro name at all specially, figuring the programmer
might want to use italics or bold face in the midst of the name.  Note
that in this implementation, programmers may only use directives in
macro names that are recognized in preformatted text elements (|PRE|).

@d Write \HTML\ macro declaration
@{
  fputs("<a name=\"nuweb", html_file);
  write_single_scrap_ref(html_file, scraps);
  fprintf(html_file, "\">&lt;%s ", name->spelling);
  write_single_scrap_ref(html_file, scraps);
  fputs("&gt;</a> =\n", html_file);
@}

@d Begin \HTML\ scrap environment
@{
{
  if (hw_flag) fputs("\\begin{rawhtml}\n", html_file);
  fputs("<pre>\n", html_file);
}
@}

The end of a scrap is marked with the characters |<>|.

@d Fill in the middle of \HTML\ scrap environment
@{
{
  copy_scrap(html_file);
  fputs("&lt;&gt;</pre>\n", html_file);
}
@}

The only task remaining is to get rid of the current at command and
end the paragraph.

@d Finish \HTML\ scrap environment
@{
{
  if (hw_flag) fputs("\\end{rawhtml}\n", html_file);
  c = source_get(); /* Get rid of current at command. */
}
@}


\subsubsection{Formatting Cross References}

@d Write \HTML\ file defs
@{
{
  if (name->defs->next) {
    fputs("File defined by ", html_file);
    print_scrap_numbers(html_file, name->defs);
    fputs("<br>\n", html_file);
  }
}
@}

@d Write \HTML\ macro defs
@{
{
  if (name->defs->next) {
    fputs("Macro defined by ", html_file);
    print_scrap_numbers(html_file, name->defs);
    fputs("<br>\n", html_file);
  }
}
@}

@d Write \HTML\ macro refs
@{
{
  if (name->uses) {
    fputs("Macro referenced in ", html_file);
    print_scrap_numbers(html_file, name->uses);
  }
  else {
    fputs("Macro never referenced.\n", html_file);
    fprintf(stderr, "%s: <%s> never referenced.\n",
	    command_name, name->spelling);
  }
  fputs("<br>\n", html_file);
}
@}

@o html.c -d
@{
static void display_scrap_ref(FILE *html_file, int num)
{
  fputs("<a href=\"#nuweb", html_file);
  write_single_scrap_ref(html_file, num);
  fputs("\">", html_file);
  write_single_scrap_ref(html_file, num);
  fputs("</a>", html_file);
}
@| display_scrap_ref @}

@o html.c -d
@{
static void display_scrap_numbers(FILE *html_file, Scrap_Node *scraps)
{
  display_scrap_ref(html_file, scraps->scrap);
  scraps = scraps->next;
  while (scraps) {
    fputs(", ", html_file);
    display_scrap_ref(html_file, scraps->scrap);
    scraps = scraps->next;
  }
}
@| display_scrap_numbers @}

@o html.c -d
@{
static void print_scrap_numbers(FILE *html_file, Scrap_Node *scraps)
{
  fputs("scrap", html_file);
  if (scraps->next) putc('s', html_file);
  putc(' ', html_file);
  display_scrap_numbers(html_file, scraps);
  fputs(".\n", html_file);
}
@| print_scrap_numbers @}


\subsubsection{Formatting a Scrap}

We must translate \HTML\ special keywords into entities in scraps.

@o html.c -d
@{
static void copy_scrap(FILE *file)
{
  int indent = 0;
  int c = source_get();
  while (1) {
    switch (c) {
      case '@@':  @<Check \HTML\ at-sequence for end-of-scrap@>
		 break;
      case '<' : fputs("&lt;", file);
		 indent++;
		 break;
      case '>' : fputs("&gt;", file);
		 indent++;
		 break;
      case '&' : fputs("&amp;", file);
		 indent++;
		 break;
      case '\n': putc(c, file);
		 indent = 0;
		 break;
      case '\t': @<Expand tab into spaces@>
		 break;
      default:   putc(c, file);
		 indent++;
		 break;
    }
    c = source_get();
  }
}
@| copy_scrap @}

@d Check \HTML\ at-sequence...
@{
{
  c = source_get();
  switch (c) {
    case '@@': putc(c, file);
	      break;
    case '|': @<Skip over index entries@>
    case '}': return;
    case '<': @<Format \HTML\ macro name@>
	      break;
    default:  /* ignore these since pass1 will have warned about them */
	      break;
  }
}
@}

There's no need to check for errors here, since we will have already
pointed out any during the first pass.

@d Format \HTML\ macro name
@{
{
  Name *name = collect_scrap_name();
  fprintf(file, "&lt;%s ", name->spelling);
  if (name->defs)
    @<Write \HTML\ abbreviated definition list@>
  else {
    putc('?', file);
    fprintf(stderr, "%s: scrap never defined <%s>\n",
	    command_name, name->spelling);
  }
  fputs("&gt;", file);
}
@}


@d Write \HTML\ abbreviated definition list
@{
{
  Scrap_Node *p = name->defs;
  display_scrap_ref(file, p->scrap);
  if (p->next)
    fputs(", ... ", file);
}
@}


\subsection{Generating the Indices}

@d Write \HTML\ index of file names
@{
{
  if (file_names) {
    if (hw_flag) fputs("\\begin{rawhtml}\n", html_file);
    fputs("<dl compact>\n", html_file);
    format_entry(file_names, html_file, TRUE);
    fputs("</dl>\n", html_file);
    if (hw_flag) fputs("\\end{rawhtml}\n", html_file);
  }
  c = source_get();
}
@}


@d Write \HTML\ index of macro names
@{
{
  if (macro_names) {
    if (hw_flag) fputs("\\begin{rawhtml}\n", html_file);
    fputs("<dl compact>\n", html_file);
    format_entry(macro_names, html_file, FALSE);
    fputs("</dl>\n", html_file);
    if (hw_flag) fputs("\\end{rawhtml}\n", html_file);
  }
  c = source_get();
}
@}


@o html.c -d
@{
static void format_entry(Name *name, FILE *html_file, int file_flag)
{
  while (name) {
    format_entry(name->llink, html_file, file_flag);
    @<Format an \HTML\ index entry@>
    name = name->rlink;
  }
}
@| format_entry @}


@d Format an \HTML\ index entry
@{
{
  fputs("<dt> ", html_file);
  if (file_flag) {
    fprintf(html_file, "<code>\"%s\"</code>\n<dd> ", name->spelling);
    @<Write \HTML\ file's defining scrap numbers@>
  }
  else {
    fprintf(html_file, "&lt;%s ", name->spelling);
    @<Write \HTML\ defining scrap numbers@>
    fputs("&gt;\n<dd> ", html_file);
    @<Write \HTML\ referencing scrap numbers@>
  }
  putc('\n', html_file);
}
@}


@d Write \HTML\ file's defining scrap numbers
@{
{
  fputs("Defined by ", html_file);
  print_scrap_numbers(html_file, name->defs);
}
@}

@d Write \HTML\ defining scrap numbers
@{
{
  if (name->defs)
    display_scrap_numbers(html_file, name->defs);
  else
    putc('?', html_file);
}
@}

@d Write \HTML\ referencing scrap numbers
@{
{
  Scrap_Node *p = name->uses;
  if (p) {
    fputs("Referenced in ", html_file);
    print_scrap_numbers(html_file, p);
  }
  else
    fputs("Not referenced.\n", html_file);
}
@}


@d Write \HTML\ index of user-specified names
@{
{
  if (user_names) {
    if (hw_flag) fputs("\\begin{rawhtml}\n", html_file);
    fputs("<dl compact>\n", html_file);
    format_user_entry(user_names, html_file);
    fputs("</dl>\n", html_file);
    if (hw_flag) fputs("\\end{rawhtml}\n", html_file);
  }
  c = source_get();
}
@}


@o html.c -d
@{
static void format_user_entry(Name *name, FILE *html_file)
{
  while (name) {
    format_user_entry(name->llink, html_file);
    @<Format a user \HTML\ index entry@>
    name = name->rlink;
  }
}
@| format_user_entry @}


@d Format a user \HTML\ index entry
@{
{
  Scrap_Node *uses = name->uses;
  if (uses) {
    Scrap_Node *defs = name->defs;
    fprintf(html_file, "<dt><code>%s</code>:\n<dd> ", name->spelling);
    if (uses->scrap < defs->scrap) {
      display_scrap_ref(html_file, uses->scrap);
      uses = uses->next;
    }
    else {
      if (defs->scrap == uses->scrap)
	uses = uses->next;
      fputs("<strong>", html_file);
      display_scrap_ref(html_file, defs->scrap);
      fputs("</strong>", html_file);
      defs = defs->next;
    }
    while (uses || defs) {
      fputs(", ", html_file);
      if (uses && (!defs || uses->scrap < defs->scrap)) {
	display_scrap_ref(html_file, uses->scrap);
	uses = uses->next;
      }
      else {
	if (uses && defs->scrap == uses->scrap)
	  uses = uses->next;
	fputs("<strong>", html_file);
	display_scrap_ref(html_file, defs->scrap);
	fputs("</strong>", html_file);
	defs = defs->next;
      }
    }
    fputs(".\n", html_file);
  }
}
@}



\section{Writing the Program Files} \label{program-files}

@d Function pro...
@{void write_files(Name *files);
@}

@o program.c -d
@{
void write_files(Name *files)
{
  while (files) {
    write_files(files->llink);
    @<Write out {\tt files->spelling}@>
    files = files->rlink;
  }
}@| write_files @}

We call |tmpnam|, causing it to create a file name in the current directory.
This could cause a problem for |rename| if the eventual program file will
reside on a different file system from that on which temporary files are
created. Perhaps it would be better to examine |files->spelling| to find any
directory information.

Note the superfluous call to |remove| before |rename|. We're using it get
around a bug in some implementations of |rename|.

@d Write out {\tt files->spelling}
@{
{
  char indent_chars[500];
  FILE *temp_file;
  char *temp_name = tmpnam(NULL);
  temp_file = fopen(temp_name, "w");
  if (!temp_file) {
    fprintf(stderr, "%s: can't create %s for a temporary file\n",
	    command_name, temp_name);
    exit(-1);
  }
  if (verbose_flag)
    fprintf(stderr, "writing %s\n", files->spelling);
  write_scraps(temp_file, files->defs, 0, indent_chars,
	       files->debug_flag, files->tab_flag, files->indent_flag);
  fclose(temp_file);
  if (compare_flag)
    @<Compare the temp file and the old file@>
  else {
    remove(files->spelling);
    rename(temp_name, files->spelling);
  }
}
@}

Again, we use a call to |remove| before |rename|.
The comparison is performed exactly if the |-C| flag is not used;
otherwise lines starting with \verb*!#line ! are ignored.

@d Compare the temp file...
@{
{
  FILE *old_file = fopen(files->spelling, "r");
  if (old_file) {
    int x, y;
    temp_file = fopen(temp_name, "r");
    if (compline_flag)
      @<Compare the files ignoring \texttt{\#line} directives@>
    else
      @<Compare the files exactly@>
    fclose(old_file);
    fclose(temp_file);
    if (x == y)
      remove(temp_name);
    else {
      remove(files->spelling);
      rename(temp_name, files->spelling);
    }
  }
  else
    rename(temp_name, files->spelling);
}
@}

@d Compare the files exactly
@{
do {
  x = getc(old_file);
  y = getc(temp_file);
} while (x == y && x != EOF);
@}

@d Compare the files ignoring \texttt{\#line} directives
@{
{
  char cue[] = "\n#line ";
  int pos = 0, cuelen = strlen(cue);
  do {
    x = getc(old_file);
    y = getc(temp_file);
    if (x != cue[pos])
      pos = 0;
    if (x == y && x == cue[pos]) {
      pos++;
      if (pos == cuelen) {
        pos = 0;
        do x = getc(old_file); while (x != '\n' && x != EOF);
        do y = getc(temp_file); while (y != '\n' && y != EOF);
      }
    }
  } while (x == y && x != EOF);
}
@}



\chapter{The Support Routines}

\section{Source Files} \label{source-files}

\subsection{Global Declarations}

We need two routines to handle reading the source files.

@d Function pro...
@{
void source_open(char *name);
int source_get(void);  /* returns next char or EOF */
int source_next(void); /* returns next char to be read by source_get() */
@}


There are also two global variables maintained for use in error
messages and such.

@d Global variable dec...
@{
extern char *source_name;  /* name of the current file */
extern int source_line;    /* current line in the source file */
@| source_name source_line @}

@d Global variable def...
@{
char *source_name = NULL;
int source_line = 0;
@}


\subsection{Local Declarations}

@o input.c -d
@{
static FILE *source_file;  /* the current input file */
static int source_peek;
static int double_at;
static int include_depth;
@| source_peek source_file double_at include_depth @}


@o input.c -d
@{
#define MAX_INCLUDES 16
static struct {
  FILE *file;
  char *name;
  int line;
} stack[MAX_INCLUDES];
@| stack @}


\subsection{Reading a File}

The routine |source_get| returns the next character from the
current source file. It notices newlines and keeps the line counter
|source_line| up to date. It also catches |EOF| and watches
for |@@|~characters. All other characters are immediately returned.

@o input.c -d
@{
int source_get(void)
{
  int c = source_peek;
  switch (c) {
    case EOF:  @<Handle {\tt EOF}@>
	       return c;
    case '@@':  @<Handle an ``at'' character@>
	       return c;
    case '\n': source_line++;
    default:   source_peek = getc(source_file);
	       return c;
  }
}
@| source_get @}

The routine |source_next| returns the next character that will be
returned by |source_get|. It performs no processing (in fact,
it just returns the value of |source_peek|).

@o input.c -d
@{
int source_next(void)
{
  return source_peek;
}
@| source_next @}

This whole |@@|~character handling mess is pretty annoying.
I want to recognize |@@i| so I can handle include files correctly.
At the same time, it makes sense to recognize illegal |@@|~sequences
and complain; this avoids ever having to check anywhere else.
Unfortunately, I need to avoid tripping over the |@@@@|~sequence;
hence this whole unsatisfactory |double_at| business.

@d Handle an ``at''...
@{
{
  c = getc(source_file);
  if (double_at) {
    source_peek = c;
    double_at = FALSE;
    c = '@@';
  }
  else
    switch (c) {
      case 'i': @<Open an include file@>
		break;
      case 'f': case 'm': case 'u':
      case 'd': case 'o': case 'D': case 'O':
      case '{': case '}': case '<': case '>': case '|':
		source_peek = c;
		c = '@@';
		break;
      case '@@': source_peek = c;
		double_at = TRUE;
		break;
      default:  fprintf(stderr, "%s: bad @@ sequence (%s, line %d)\n",
			command_name, source_name, source_line);
		exit(-1);
    }
}
@}

@d Open an include file
@{
{
  char name[256];
  if (include_depth >= MAX_INCLUDES) {
    fprintf(stderr, "%s: include nesting too deep (%s, %d)\n",
	    command_name, source_name, source_line);
    exit(-1);
  }
  @<Collect include-file name@>
  stack[include_depth].name = source_name;
  stack[include_depth].file = source_file;
  stack[include_depth].line = source_line + 1;
  include_depth++;
  source_line = 1;
  source_name = save_string(name);
  source_file = fopen(source_name, "r");
  if (!source_file) {
    fprintf(stderr, "%s: can't open include file %s\n",
     command_name, source_name);
    exit(-1);
  }
  source_peek = getc(source_file);
  c = source_get();
}
@}

@d Collect include-file name
@{
{
    char *p = name;
    do
      c = getc(source_file);
    while (c == ' ' || c == '\t');
    while (isgraph(c)) {
      *p++ = c;
      c = getc(source_file);
    }
    *p = '\0';
    if (c != '\n') {
      fprintf(stderr, "%s: unexpected characters after file name (%s, %d)\n",
	      command_name, source_name, source_line);
      exit(-1);
    }
}
@}

If an |EOF| is discovered, the current file must be closed and
input from the next stacked file must be resumed. If no more files are
on the stack, the |EOF| is returned.

@d Handle {\tt EOF}
@{
{
  fclose(source_file);
  if (include_depth) {
    include_depth--;
    source_file = stack[include_depth].file;
    source_line = stack[include_depth].line;
    source_name = stack[include_depth].name;
    source_peek = getc(source_file);
    c = source_get();
  }
}
@}


\subsection{Opening a File}

The routine |source_open| takes a file name and tries to open the
file. If unsuccessful, it complains and halts. Otherwise, it sets
|source_name|, |source_line|, and |double_at|.

@o input.c -d
@{
void source_open(char *name)
{
  source_file = fopen(name, "r");
  if (!source_file) {
    fprintf(stderr, "%s: couldn't open %s\n", command_name, name);
    exit(-1);
  }
  source_name = name;
  source_line = 1;
  source_peek = getc(source_file);
  double_at = FALSE;
  include_depth = 0;
}
@| source_open @}



\section{Scraps} \label{scraps}

@o scraps.c -d
@{
#define SLAB_SIZE 500

typedef struct slab {
  struct slab *next;
  char chars[SLAB_SIZE];
} Slab;
@| Slab SLAB_SIZE @}


@o scraps.c -d
@{
typedef struct {
  char *file_name;
  int file_line;
  int page;
  char letter;
  Slab *slab;
} ScrapEntry;
@| ScrapEntry @}

@o scraps.c -d
@{
static ScrapEntry *SCRAP[256];

#define scrap_array(i) SCRAP[(i) >> 8][(i) & 255]

static int scraps;
@| scraps scrap_array SCRAP @}


@d Function pro...
@{
void init_scraps(void);
int collect_scrap(void);
int write_scraps(FILE *file, Scrap_Node *defs, int global_indent,
     char *indent_chars, char debug_flag, char tab_flag, char indent_flag);
void write_scrap_ref(FILE *file, int num, int first, int *page);
void write_single_scrap_ref(FILE *file, int num);
@}


@o scraps.c -d
@{
void init_scraps(void)
{
  scraps = 1;
  SCRAP[0] = (ScrapEntry *) arena_getmem(256 * sizeof(ScrapEntry));
}
@| init_scraps @}


@o scraps.c -d
@{
void write_scrap_ref(FILE *file, int num, int first, int *page)
{
  if (scrap_array(num).page >= 0) {
    if (first)
      fprintf(file, "%d", scrap_array(num).page);
    else if (scrap_array(num).page != *page)
      fprintf(file, ", %d", scrap_array(num).page);
    if (scrap_array(num).letter > 0)
      putc(scrap_array(num).letter, file);
  }
  else {
    if (first)
      putc('?', file);
    else
      fputs(", ?", file);
    @<Warn (only once) about needing to rerun after \LaTeX@>
  }
  *page = scrap_array(num).page;
}
@| write_scrap_ref @}

@o scraps.c -d
@{
void write_single_scrap_ref(FILE *file, int num)
{
  int page;
  write_scrap_ref(file, num, TRUE, &page);
}
@| write_single_scrap_ref @}


@d Warn (only once) about needing to...
@{
{
  if (!already_warned) {
    fprintf(stderr, "%s: you'll need to rerun nuweb after running LaTeX\n",
	    command_name);
    already_warned = TRUE;
  }
}
@}

@d Global variable dec...
@{extern int already_warned;
@| already_warned @}

@d Global variable def...
@{int already_warned = 0;
@}

@o scraps.c -d
@{
typedef struct {
  Slab *scrap;
  Slab *prev;
  int index;
} Manager;
@| Manager @}



@o scraps.c -d
@{
static void push(char c, Manager *manager)
{
  Slab *scrap = manager->scrap;
  int index = manager->index;
  scrap->chars[index++] = c;
  if (index == SLAB_SIZE) {
    Slab *new = (Slab *) arena_getmem(sizeof(Slab));
    scrap->next = new;
    manager->scrap = new;
    index = 0;
  }
  manager->index = index;
}
@| push @}

@o scraps.c -d
@{
static void pushs(char *s, Manager *manager)
{
  while (*s)
    push(*s++, manager);
}
@| pushs @}


@o scraps.c -d
@{
int collect_scrap(void)
{
  Manager writer;
  @<Create new scrap...@>
  @<Accumulate scrap and return {\tt scraps++}@>
}
@| collect_scrap @}

@d Create new scrap, managed by {\tt writer}
@{
{
  Slab *scrap = (Slab *) arena_getmem(sizeof(Slab));
  if ((scraps & 255) == 0)
    SCRAP[scraps >> 8] = (ScrapEntry *) arena_getmem(256 * sizeof(ScrapEntry));
  scrap_array(scraps).slab = scrap;
  scrap_array(scraps).file_name = save_string(source_name);
  scrap_array(scraps).file_line = source_line;
  scrap_array(scraps).page = -1;
  scrap_array(scraps).letter = 0;
  writer.scrap = scrap;
  writer.index = 0;
}
@}


@d Accumulate scrap...
@{
{
  int c = source_get();
  while (1) {
    switch (c) {
      case EOF: fprintf(stderr, "%s: unexpect EOF in scrap (%s, %d)\n",
			command_name, scrap_array(scraps).file_name,
			scrap_array(scraps).file_line);
		exit(-1);
      case '@@': @<Handle at-sign during scrap accumulation@>
		break;
      default:  push(c, &writer);
		c = source_get();
		break;
    }
  }
}
@}


@d Handle at-sign during scrap accumulation
@{
{
  c = source_get();
  switch (c) {
    case '@@': pushs("@@@@", &writer);
	      c = source_get();
	      break;
    case '|': @<Collect user-specified index entries@>
    case '}': push('\0', &writer);
	      return scraps++;
    case '<': @<Handle macro invocation in scrap@>
	      break;
    default : fprintf(stderr, "%s: unexpected @@%c in scrap (%s, %d)\n",
		      command_name, c, source_name, source_line);
	      exit(-1);
  }
}
@}


@d Collect user-specified index entries
@{
{
  do {
    char new_name[256];
    char *p = new_name;
    do
      c = source_get();
    while (isspace(c));
    if (c != '@@') {
      Name *name;
      do {
	*p++ = c;
	c = source_get();
      } while (c != '@@' && !isspace(c));
      *p = '\0';
      name = name_add(&user_names, new_name);
      if (!name->defs || name->defs->scrap != scraps) {
	Scrap_Node *def = (Scrap_Node *) arena_getmem(sizeof(Scrap_Node));
	def->scrap = scraps;
	def->next = name->defs;
	name->defs = def;
      }
    }
  } while (c != '@@');
  c = source_get();
  if (c != '}') {
    fprintf(stderr, "%s: unexpected @@%c in scrap (%s, %d)\n",
	    command_name, c, source_name, source_line);
    exit(-1);
  }
}
@}


@d Handle macro invocation in scrap
@{
{
  Name *name = collect_scrap_name();
  @<Save macro name@>
  @<Add current scrap to {\tt name}'s uses@>
  c = source_get();
}
@}


@d Save macro name
@{
{
  char *s = name->spelling;
  int len = strlen(s) - 1;
  pushs("@@<", &writer);
  while (len > 0) {
    push(*s++, &writer);
    len--;
  }
  if (*s == ' ')
    pushs("...", &writer);
  else
    push(*s, &writer);
  pushs("@@>", &writer);
}
@}


@d Add current scrap to...
@{
{
  if (!name->uses || name->uses->scrap != scraps) {
    Scrap_Node *use = (Scrap_Node *) arena_getmem(sizeof(Scrap_Node));
    use->scrap = scraps;
    use->next = name->uses;
    name->uses = use;
  }
}
@}


@o scraps.c -d
@{
static char pop(Manager *manager)
{
  Slab *scrap = manager->scrap;
  int index = manager->index;
  char c = scrap->chars[index++];
  if (index == SLAB_SIZE) {
    manager->prev = scrap;
    manager->scrap = scrap->next;
    index = 0;
  }
  manager->index = index;
  return c;
}
@| pop @}



@o scraps.c -d
@{
static Name *pop_scrap_name(Manager *manager)
{
  char name[256];
  char *p = name;
  int c = pop(manager);
  while (TRUE) {
    if (c == '@@')
      @<Check for end of scrap name and return@>
    else {
      *p++ = c;
      c = pop(manager);
    }
  }
}
@| pop_scrap_name @}


@d Check for end of scrap name...
@{
{
  c = pop(manager);
  if (c == '@@') {
    *p++ = c;
    c = pop(manager);
  }
  else if (c == '>') {
    if (p - name > 3 && p[-1] == '.' && p[-2] == '.' && p[-3] == '.') {
      p[-3] = ' ';
      p -= 2;
    }
    *p = '\0';
    return prefix_add(&macro_names, name);
  }
  else {
    fprintf(stderr, "%s: found an internal problem\n", command_name);
    exit(-1);
  }
}
@}


@o scraps.c -d
@{
int write_scraps(FILE *file, Scrap_Node *defs, int global_indent,
     char *indent_chars, char debug_flag, char tab_flag, char indent_flag)
{
  int indent = 0;
  while (defs) {
    @<Copy {\tt defs->scrap} to {\tt file}@>
    defs = defs->next;
  }
  return indent + global_indent;
}
@| write_scraps @}


@d Copy {\tt defs->scrap...
@{
{
  char c;
  Manager reader;
  int line_number = scrap_array(defs->scrap).file_line;
  @<Insert debugging information if required@>
  reader.scrap = scrap_array(defs->scrap).slab;
  reader.index = 0;
  c = pop(&reader);
  while (c) {
    switch (c) {
      case '@@':  @<Check for macro invocation in scrap@>
		 break;
      case '\n': putc(c, file);
		 line_number++;
		 @<Insert appropriate indentation@>
		 break;
      case '\t': @<Handle tab...@>
		 break;
      default:	 putc(c, file);
		 indent_chars[global_indent + indent] = ' ';
		 indent++;
		 break;
    }
    c = pop(&reader);
  }
}
@}


@d Insert debugging information if required
@{
if (debug_flag) {
  fprintf(file, "\n#line %d \"%s\"\n",
	  line_number, scrap_array(defs->scrap).file_name);
  @<Insert appropr...@>
}
@}


@d Insert approp...
@{
{
  if (indent_flag) {
    if (tab_flag)
      for (indent=0; indent<global_indent; indent++)
	putc(' ', file);
    else
      for (indent=0; indent<global_indent; indent++)
	putc(indent_chars[indent], file);
  }
  indent = 0;
}
@}


@d Handle tab characters on output
@{
{
  if (tab_flag)
    @<Expand tab...@>
  else {
    putc('\t', file);
    indent_chars[global_indent + indent] = '\t';
    indent++;
  }
}
@}



@d Check for macro invocation...
@{
{
  c = pop(&reader);
  switch (c) {
    case '@@': putc(c, file);
	      indent_chars[global_indent + indent] = ' ';
	      indent++;
	      break;
    case '<': @<Copy macro into {\tt file}@>
	      @<Insert debugging information if required@>
	      break;
    default:  /* ignore, since we should already have a warning */
	      break;
  }
}
@}


@d Copy macro into...
@{
{
  Name *name = pop_scrap_name(&reader);
  if (name->mark) {
    fprintf(stderr, "%s: recursive macro discovered involving <%s>\n",
	    command_name, name->spelling);
    exit(-1);
  }
  if (name->defs) {
    name->mark = TRUE;
    indent = write_scraps(file, name->defs, global_indent + indent,
			  indent_chars, debug_flag, tab_flag, indent_flag);
    indent -= global_indent;
    name->mark = FALSE;
  }
  else if (!tex_flag)
    fprintf(stderr, "%s: macro never defined <%s>\n",
	    command_name, name->spelling);
}
@}


\subsection{Collecting Page Numbers}

@d Function...
@{void collect_numbers(char *aux_name);
@}

@o scraps.c -d
@{
void collect_numbers(char *aux_name)
{
  if (number_flag) {
    int i;
    for (i=1; i<scraps; i++)
      scrap_array(i).page = i;
  }
  else {
    FILE *aux_file = fopen(aux_name, "r");
    already_warned = FALSE;
    if (aux_file) {
      char aux_line[500];
      while (fgets(aux_line, 500, aux_file)) {
        int scrap_number;
        int page_number;
        char dummy[50];
        if (3 == sscanf(aux_line, "\\newlabel{scrap%d}{%[^}]}{%d}",
			&scrap_number, dummy, &page_number)) {
	  if (scrap_number < scraps)
	    scrap_array(scrap_number).page = page_number;
	  else
	    @<Warn...@>
        }
      }
      fclose(aux_file);
      @<Add letters to scraps with duplicate page numbers@>
    }
  }
}
@| collect_numbers @}

@d Add letters to scraps with...
@{
{
  int scrap;
  for (scrap=2; scrap<scraps; scrap++) {
    if (scrap_array(scrap-1).page == scrap_array(scrap).page) {
      if (!scrap_array(scrap-1).letter)
        scrap_array(scrap-1).letter = 'a';
      scrap_array(scrap).letter = scrap_array(scrap-1).letter + 1;
    }
  }
}
@}



\section{Names} \label{names}

@d Type de...
@{
typedef struct scrap_node {
  struct scrap_node *next;
  int scrap;
} Scrap_Node;
@| Scrap_Node @}


@d Type de...
@{
typedef struct name {
  char *spelling;
  struct name *llink;
  struct name *rlink;
  Scrap_Node *defs;
  Scrap_Node *uses;
  int mark;
  char tab_flag;
  char indent_flag;
  char debug_flag;
} Name;
@| Name @}

@d Global variable dec...
@{
extern Name *file_names;
extern Name *macro_names;
extern Name *user_names;
@| file_names macro_names user_names @}

@d Global variable def...
@{
Name *file_names = NULL;
Name *macro_names = NULL;
Name *user_names = NULL;
@}

@d Function pro...
@{
Name *collect_file_name(void);
Name *collect_macro_name(void);
Name *collect_scrap_name(void);
Name *name_add(Name *root[], char *spelling);
Name *prefix_add(Name *root[], char *spelling);
char *save_string(char *s);
void reverse_lists(Name *names);
@}

@o names.c -d
@{
enum { LESS, GREATER, EQUAL, PREFIX, EXTENSION };

static int compare(char *x, char *y)
{
  int len, result;
  int xl = strlen(x);
  int yl = strlen(y);
  int xp = x[xl - 1] == ' ';
  int yp = y[yl - 1] == ' ';
  if (xp) xl--;
  if (yp) yl--;
  len = xl < yl ? xl : yl;
  result = strncmp(x, y, len);
  if (result < 0) return GREATER;
  else if (result > 0) return LESS;
  else if (xl < yl) {
    if (xp) return EXTENSION;
    else return LESS;
  }
  else if (xl > yl) {
    if (yp) return PREFIX;
    else return GREATER;
  }
  else return EQUAL;
}
@| compare LESS GREATER EQUAL PREFIX EXTENSION @}


@o names.c -d
@{
char *save_string(char *s)
{
  char *new = (char *) arena_getmem((strlen(s) + 1) * sizeof(char));
  strcpy(new, s);
  return new;
}
@| save_string @}

@o names.c -d
@{
static int ambiguous_prefix(Name *node, char *spelling);

Name *prefix_add(Name *root[], char *spelling)
{
  Name *node = *root;
  while (node) {
    switch (compare(node->spelling, spelling)) {
    case GREATER:   root = &node->rlink;
		    break;
    case LESS:      root = &node->llink;
		    break;
    case EQUAL:     return node;
    case EXTENSION: node->spelling = save_string(spelling);
		    return node;
    case PREFIX:    @<Check for ambiguous prefix@>
		    return node;
    }
    node = *root;
  }
  @<Create new name entry@>
}
@| prefix_add @}

Since a very short prefix might match more than one macro name, I need
to check for other matches to avoid mistakes, so I continue the search down
\emph{both} branches of the tree.

@d Check for ambiguous prefix
@{
{
  if (ambiguous_prefix(node->llink, spelling) ||
      ambiguous_prefix(node->rlink, spelling))
    fprintf(stderr,
	    "%s: ambiguous prefix @@<%s...@@> (%s, line %d)\n",
	    command_name, spelling, source_name, source_line);
}
@}

@o names.c -d
@{
static int ambiguous_prefix(Name *node, char *spelling)
{
  while (node) {
    switch (compare(node->spelling, spelling)) {
    case GREATER:   node = node->rlink;
		    break;
    case LESS:      node = node->llink;
		    break;
    case EQUAL:
    case EXTENSION:
    case PREFIX:    return TRUE;
    }
  }
  return FALSE;
}
@}

Rob Shillingsburg suggested that I organize the index of
user-specified identifiers more traditionally; that is, not relying on
strict {\small ASCII} comparisons via |strcmp|. Ideally, we'd like
to see the index ordered like this:
\begin{quote}
\begin{flushleft}
aardvark \\
Adam \\
atom \\
Atomic \\
atoms
\end{flushleft}
\end{quote}
The function |robs_strcmp| implements the desired predicate.

@o names.c -d
@{
static int robs_strcmp(char *x, char *y)
{
  char *xx = x;
  char *yy = y;
  int xc = toupper(*xx);
  int yc = toupper(*yy);
  while (xc == yc && xc) {
    xx++;
    yy++;
    xc = toupper(*xx);
    yc = toupper(*yy);
  }
  if (xc != yc) return xc - yc;
  xc = *x;
  yc = *y;
  while (xc == yc && xc) {
    x++;
    y++;
    xc = *x;
    yc = *y;
  }
  if (isupper(xc) && islower(yc))
    return xc * 2 - (toupper(yc) * 2 + 1);
  if (islower(xc) && isupper(yc))
    return toupper(xc) * 2 + 1 - yc * 2;
  return xc - yc;
}
@| robs_strcmp @}

@o names.c -d
@{
Name *name_add(Name *root[], char *spelling)
{
  Name *node = *root;
  while (node) {
    int result = robs_strcmp(node->spelling, spelling);
    if (result > 0)
      root = &node->llink;
    else if (result < 0)
      root = &node->rlink;
    else
      return node;
    node = *root;
  }
  @<Create new name entry@>
}
@| name_add @}


@d Create new name...
@{
{
  node = (Name *) arena_getmem(sizeof(Name));
  node->spelling = save_string(spelling);
  node->mark = FALSE;
  node->llink = NULL;
  node->rlink = NULL;
  node->uses = NULL;
  node->defs = NULL;
  node->tab_flag = TRUE;
  node->indent_flag = TRUE;
  node->debug_flag = FALSE;
  *root = node;
  return node;
}
@}


Name terminated by whitespace.  Also check for per-file flags. Keep
skipping white space until we reach the scrap.

@o names.c -d
@{
Name *collect_file_name(void)
{
  Name *new_name;
  char name[256];
  char *p = name;
  int start_line = source_line;
  int c = source_get();
  while (isspace(c))
    c = source_get();
  while (isgraph(c)) {
    *p++ = c;
    c = source_get();
  }
  if (p == name) {
    fprintf(stderr, "%s: expected file name (%s, %d)\n",
	    command_name, source_name, start_line);
    exit(-1);
  }
  *p = '\0';
  new_name = name_add(&file_names, name);
  @<Handle optional per-file flags@>
  if (c != '@@' || source_get() != '{') {
    fprintf(stderr, "%s: expected @@{ after file name (%s, %d)\n",
	    command_name, source_name, start_line);
    exit(-1);
  }
  if (skipnl_flag && source_next() == '\n') source_get();
  return new_name;
}
@| collect_file_name @}

@d Handle optional per-file flags
@{
{
  while (1) {
    while (isspace(c))
      c = source_get();
    if (c == '-') {
      c = source_get();
      do {
	switch (c) {
	  case 't': new_name->tab_flag = FALSE;
		    break;
	  case 'd': new_name->debug_flag = TRUE;
		    break;
	  case 'i': new_name->indent_flag = FALSE;
		    break;
	  default : fprintf(stderr, "%s: unexpected per-file flag (%s, %d)\n",
			    command_name, source_name, source_line);
		    break;
	}
	c = source_get();
      } while (!isspace(c));
    }
    else break;
  }
}
@}

Name terminated by |\n| or |@@{|; but keep skipping until |@@{|.

@o names.c -d
@{
Name *collect_macro_name(void)
{
  char name[256];
  char *p = name;
  int start_line = source_line;
  int c = source_get();
  while (isspace(c))
    c = source_get();
  while (c != EOF) {
    switch (c) {
      case '@@':  @<Check for terminating at-sequence and return name@>
		 break;
      case '\t':
      case ' ':  *p++ = ' ';
		 do
		   c = source_get();
		 while (c == ' ' || c == '\t');
		 break;
      case '\n': @<Skip until scrap begins, then return name@>
      default:	 *p++ = c;
		 c = source_get();
		 break;
    }
  }
  fprintf(stderr, "%s: expected macro name (%s, %d)\n",
	  command_name, source_name, start_line);
  exit(-1);
  return NULL;  /* unreachable return to avoid warnings on some compilers */
}
@| collect_macro_name @}

@d Check for termina...
@{
{
  c = source_get();
  switch (c) {
    case '@@': *p++ = c;
	      break;
    case '{': @<Cleanup and install name@>
    default:  fprintf(stderr,
		      "%s: unexpected @@%c in macro name (%s, %d)\n",
		      command_name, c, source_name, start_line);
	      exit(-1);
  }
}
@}

@d Cleanup and install name
@{
{
  if (p > name && p[-1] == ' ')
    p--;
  if (p - name > 3 && p[-1] == '.' && p[-2] == '.' && p[-3] == '.') {
    p[-3] = ' ';
    p -= 2;
  }
  if (p == name || name[0] == ' ') {
    fprintf(stderr, "%s: empty scrap name (%s, %d)\n",
	    command_name, source_name, source_line);
    exit(-1);
  }
  *p = '\0';
  return prefix_add(&macro_names, name);
}
@}

@d Skip until scrap...
@{
{
  do
    c = source_get();
  while (isspace(c));
  if (c != '@@' || source_get() != '{') {
    fprintf(stderr, "%s: expected @@{ after macro name (%s, %d)\n",
	    command_name, source_name, start_line);
    exit(-1);
  }
  if (skipnl_flag && source_next() == '\n') source_get();
  @<Cleanup and install name@>
}
@}

Terminated by |@@>|

@o names.c -d
@{
Name *collect_scrap_name(void)
{
  char name[256];
  char *p = name;
  int c = source_get();
  while (c == ' ' || c == '\t')
    c = source_get();
  while (c != EOF) {
    switch (c) {
      case '@@':  @<Look for end of scrap name and return@>
		 break;
      case '\t':
      case ' ':  *p++ = ' ';
		 do
		   c = source_get();
		 while (c == ' ' || c == '\t');
		 break;
      default:	 if (!isgraph(c)) {
		   fprintf(stderr,
			   "%s: unexpected character in macro name (%s, %d)\n",
			   command_name, source_name, source_line);
		   exit(-1);
		 }
		 *p++ = c;
		 c = source_get();
		 break;
    }
  }
  fprintf(stderr, "%s: unexpected end of file (%s, %d)\n",
	  command_name, source_name, source_line);
  exit(-1);
  return NULL;  /* unreachable return to avoid warnings on some compilers */
}
@| collect_scrap_name @}


@d Look for end of scrap name...
@{
{
  c = source_get();
  switch (c) {
    case '@@': *p++ = c;
	      c = source_get();
	      break;
    case '>': @<Cleanup and install name@>
    default:  fprintf(stderr,
		      "%s: unexpected @@%c in macro name (%s, %d)\n",
		      command_name, c, source_name, source_line);
	      exit(-1);
  }
}
@}


@o names.c -d
@{
static Scrap_Node *reverse(Scrap_Node *a);	/* a forward declaration */

void reverse_lists(Name *names)
{
  while (names) {
    reverse_lists(names->llink);
    names->defs = reverse(names->defs);
    names->uses = reverse(names->uses);
    names = names->rlink;
  }
}
@| reverse_lists @}

Just for fun, here's a non-recursive version of the traditional list
reversal code. Note that it reverses the list in place; that is, it
does no new allocations.

@o names.c -d
@{
static Scrap_Node *reverse(Scrap_Node *a)
{
  if (a) {
    Scrap_Node *b = a->next;
    a->next = NULL;
    while (b) {
      Scrap_Node *c = b->next;
      b->next = a;
      a = b;
      b = c;
    }
  }
  return a;
}
@| reverse @}



\section{Searching for Index Entries} \label{search}

Given the array of scraps and a set of index entries, we need to
search all the scraps for occurrences of each entry. The obvious
approach to this problem would be quite expensive for large documents;
however, there is an interesting  paper describing an efficient
solution~\cite{aho:75}.

@o scraps.c -d
@{
typedef struct name_node {
  struct name_node *next;
  Name *name;
} Name_Node;
@| Name_Node @}

@o scraps.c -d
@{
typedef struct goto_node {
  Name_Node *output;		/* list of words ending in this state */
  struct move_node *moves;	/* list of possible moves */
  struct goto_node *fail;	/* and where to go when no move fits */
  struct goto_node *next;	/* next goto node with same depth */
} Goto_Node;
@| Goto_Node @}

@o scraps.c -d
@{
typedef struct move_node {
  struct move_node *next;
  Goto_Node *state;
  char c;
} Move_Node;
@| Move_Node @}

@o scraps.c -d
@{
static Goto_Node *root[CHAR_MAX];
static int max_depth;
static Goto_Node **depths;
@| root max_depth depths @}


@o scraps.c -d
@{
static Goto_Node *goto_lookup(char c, Goto_Node *g)
{
  Move_Node *m = g->moves;
  while (m && m->c != c)
    m = m->next;
  if (m)
    return m->state;
  else
    return NULL;
}
@| goto_lookup @}


\subsection{Building the Automata}

@d Function pro...
@{void search(void);
@}

@o scraps.c -d
@{
static void build_gotos(Name *tree);
static int reject_match(Name *name, char post, Manager *reader);

void search(void)
{
  int i;
  for (i=0; i<CHAR_MAX; i++)
    root[i] = NULL;
  max_depth = 16;
  depths = (Goto_Node **) arena_getmem(max_depth * sizeof(Goto_Node *));
  for (i=0; i<max_depth; i++)
    depths[i] = NULL;
  build_gotos(user_names);
  @<Build failure functions@>
  @<Search scraps@>
}
@| search @}



@o scraps.c -d
@{
static void build_gotos(Name *tree)
{
  while (tree) {
    @<Extend goto graph with {\tt tree->spelling}@>
    build_gotos(tree->rlink);
    tree = tree->llink;
  }
}
@| build_gotos @}

@d Extend goto...
@{
{
  int depth = 2;
  char *p = tree->spelling;
  char c = *p++;
  Goto_Node *q = root[c];
  if (!q) {
    q = (Goto_Node *) arena_getmem(sizeof(Goto_Node));
    root[c] = q;
    q->moves = NULL;
    q->fail = NULL;
    q->moves = NULL;
    q->output = NULL;
    q->next = depths[1];
    depths[1] = q;
  }
  while (c = *p++) {
    Goto_Node *new = goto_lookup(c, q);
    if (!new) {
      Move_Node *new_move = (Move_Node *) arena_getmem(sizeof(Move_Node));
      new = (Goto_Node *) arena_getmem(sizeof(Goto_Node));
      new->moves = NULL;
      new->fail = NULL;
      new->moves = NULL;
      new->output = NULL;
      new_move->state = new;
      new_move->c = c;
      new_move->next = q->moves;
      q->moves = new_move;
      if (depth == max_depth) {
	int i;
	Goto_Node **new_depths =
	    (Goto_Node **) arena_getmem(2*depth*sizeof(Goto_Node *));
	max_depth = 2 * depth;
	for (i=0; i<depth; i++)
	  new_depths[i] = depths[i];
	depths = new_depths;
	for (i=depth; i<max_depth; i++)
	  depths[i] = NULL;
      }
      new->next = depths[depth];
      depths[depth] = new;
    }
    q = new;
    depth++;
  }
  q->output = (Name_Node *) arena_getmem(sizeof(Name_Node));
  q->output->next = NULL;
  q->output->name = tree;
}
@}


@d Build failure functions
@{
{
  int depth;
  for (depth=1; depth<max_depth; depth++) {
    Goto_Node *r = depths[depth];
    while (r) {
      Move_Node *m = r->moves;
      while (m) {
	char a = m->c;
	Goto_Node *s = m->state;
	Goto_Node *state = r->fail;
	while (state && !goto_lookup(a, state))
	  state = state->fail;
	if (state)
	  s->fail = goto_lookup(a, state);
	else
	  s->fail = root[a];
	if (s->fail) {
	  Name_Node *p = s->fail->output;
	  while (p) {
	    Name_Node *q = (Name_Node *) arena_getmem(sizeof(Name_Node));
	    q->name = p->name;
	    q->next = s->output;
	    s->output = q;
	    p = p->next;
	  }
	}
	m = m->next;
      }
      r = r->next;
    }
  }
}
@}


\subsection{Searching the Scraps}

@d Search scraps
@{
{
  for (i=1; i<scraps; i++) {
    char c;
    Manager reader;
    Goto_Node *state = NULL;
    reader.prev = NULL;
    reader.scrap = scrap_array(i).slab;
    reader.index = 0;
    c = pop(&reader);
    while (c) {
      while (state && !goto_lookup(c, state))
	state = state->fail;
      if (state)
	state = goto_lookup(c, state);
      else
	state = root[c];
      c = pop(&reader);
      if (state && state->output) {
	Name_Node *p = state->output;
	do {
	  Name *name = p->name;
	  if (!reject_match(name, c, &reader) &&
	      (!name->uses || name->uses->scrap != i)) {
	    Scrap_Node *new_use =
		(Scrap_Node *) arena_getmem(sizeof(Scrap_Node));
	    new_use->scrap = i;
	    new_use->next = name->uses;
	    name->uses = new_use;
	  }
	  p = p->next;
	} while (p);
      }
    }
  }
}
@}

\subsubsection{Rejecting Matches}

A problem with simple substring matching is that the string ``he'' would
match longer strings like ``she'' and ``her.'' Norman Ramsey suggested
examining the characters occurring immediately before and after a match and
rejecting the match if it appears to be part of a longer token. Of course,
the concept of \emph{token} is language-dependent, so we may be occasionally
mistaken. For the present, we'll consider the mechanism an experiment.

@o scraps.c -d
@{
#define sym_char(c) (isalnum(c) || (c) == '_')

static int op_char(char c)
{
  switch (c) {
    case '!': case '@@': case '#': case '%': case '$': case '^':
    case '&': case '*': case '-': case '+': case '=': case '/':
    case '|': case '~': case '<': case '>':
      return TRUE;
    default:
      return FALSE;
  }
}
@| sym_char op_char @}

@o scraps.c -d
@{
static int reject_match(Name *name, char post, Manager *reader)
{
  int len = strlen(name->spelling);
  char first = name->spelling[0];
  char last = name->spelling[len - 1];
  char prev = '\0';
  len = reader->index - len - 2;
  if (len >= 0)
    prev = reader->scrap->chars[len];
  else if (reader->prev)
    prev = reader->scrap->chars[SLAB_SIZE - len];
  if (sym_char(last) && sym_char(post)) return TRUE;
  if (sym_char(first) && sym_char(prev)) return TRUE;
  if (op_char(last) && op_char(post)) return TRUE;
  if (op_char(first) && op_char(prev)) return TRUE;
  return FALSE;
}
@| reject_match @}


\section{Memory Management} \label{memory-management}

I manage memory using a simple scheme inspired by Hanson's idea of {\em
arenas\/}~\cite{hanson:90}. Basically, I allocate all the storage required
when processing a source file (primarily for names and scraps) using calls to
|arena_getmem(n)|, where |n| specifies the number of bytes to be
allocated. When the storage is no longer required, the entire arena is freed
with a single call to  |arena_free()|. Both operations are quite fast.

@d Function p...
@{
void *arena_getmem(size_t n);
void arena_free(void);
@}

@o arena.c
@{
typedef struct chunk {
  struct chunk *next;
  char *limit;
  char *avail;
} Chunk;
@| Chunk @}

We define an empty chunk called |first|. The variable |arena|
points at the current chunk of memory; it's initially pointed at
|first|. As soon as some storage is required, a ``real'' chunk of memory
will be allocated and attached to |first->next|; storage will be
allocated from the new chunk (and later chunks if necessary).

@o arena.c
@{
static Chunk first = { NULL, NULL, NULL };
static Chunk *arena = &first;
@| first arena @}


\subsection{Allocating Memory}

The routine |arena_getmem(n)| returns a pointer to (at least) |n|
bytes of memory. Note that |n| is rounded up to ensure that returned
pointers are always aligned.  We align to the nearest 8~byte segment, since
that'll satisfy the more common 2-byte and 4-byte alignment restrictions too.

@o arena.c
@{
void *arena_getmem(size_t n)
{
  char *q;
  char *p = arena->avail;
  n = (n + 7) & ~7;		/* ensuring alignment to 8 bytes */
  q = p + n;
  if (q <= arena->limit) {
    arena->avail = q;
    return p;
  }
  @<Find a new chunk of memory@>
}
@| arena_getmem @}

If the current chunk doesn't have adequate space (at least |n| bytes) we
examine the rest of the list of chunks (starting at |arena->next|)
looking for a chunk with adequate space. If |n| is very large, we may
not find it right away or we may not find a suitable chunk at all.

@d Find a new chunk...
@{
{
  Chunk *ap = arena;
  Chunk *np = ap->next;
  while (np) {
    char *v = sizeof(Chunk) + (char *) np;
    if (v + n <= np->limit) {
      np->avail = v + n;
      arena = np;
      return v;
    }
    ap = np;
    np = ap->next;
  }
  @<Allocate a new chunk of memory@>
}
@}

If there isn't a suitable chunk of memory on the free list, then we
need to allocate a new one.

@d Allocate a new ch...
@{
{
  size_t m = n + 16384;
  np = (Chunk *) malloc(m);
  np->limit = m + (char *) np;
  np->avail = n + sizeof(Chunk) + (char *) np;
  np->next = NULL;
  ap->next = np;
  arena = np;
  return sizeof(Chunk) + (char *) np;
}
@}


\subsection{Freeing Memory}

To free all the memory in the arena, we need only point |arena| back to
the first empty chunk.

@o arena.c
@{
void arena_free(void)
{
  arena = &first;
}
@| arena_free @}



\chapter{Formatting scraps} \label{formattingscraps}

To give the user full control over how scraps are formatted, nuweb only
outputs commands defined by the \textsf{nuweb} package. By redefining these
commands, the user can control how scraps are formatted.

The package is simple; of the three packages it uses, two are part of the
standard \LaTeXe\ distribution, and the third, \textsf{multicol}, is
commonly used and available on most installations.

@o nuweb.sty
@{
% nuweb.sty
% for use with nuweb v0.90
% by Reuben Thomas

\NeedsTeXFormat{LaTeX2e}
\ProvidesPackage{nuweb}
    [1997/05/08 v1.0 nuweb support]

\RequirePackage{ifthen}
\RequirePackage{alltt}
\RequirePackage{multicol}
@}

First, we define a skeleton version of the environment in which scraps are
formatted. This takes eight arguments:
\begin{center}
|\begin{nuweb}{|\textit{top-cmd}|}{|\textit{bottom-cmd}|}{|%
\textit{title}|}{|\textit{number}|}{|\textit{refs}|}{|%
\textit{num-refs}|}{|\textit{defs}|}{|\textit{num-defs}|}|
\end{center}
The arguments are as follows:

\begin{description}
\item[\it top-cmd] is the command used to typeset the top of the scrap.
\item[\it bottom-cmd] is the command used to typeset the bottom of the scrap.
\item[\it title] is the name of the scrap.
\item[\it number] is the number of the scrap.
\item[\it refs] is the list of scraps which refer to the current scrap.
\item[\it num-refs] is the number of referring scraps.
\item[\it defs] is the list of scraps which define the current scrap.
\item[\it num-defs] is the number of defining scraps.
\end{description}

The environment \texttt{nuweb} is only a framework. It relies on the
environment \texttt{nuwebscrapcontents} to lay out the code in the scrap, and
provides no spacing around the scrap.

@o nuweb.sty
@{
\newcommand{\nuwebscrapbottomcommand}{\relax}
\newcommand{\nuwebscraptitle}{\relax}
\newcommand{\nuwebscrapnumber}{\relax}
\newcommand{\nuwebscrapdeflist}{\relax}
\newcommand{\nuwebscrapdefs}{\relax}
\newcommand{\nuwebscrapreflist}{\relax}
\newcommand{\nuwebscraprefs}{\relax}
\newcommand{\nuwebscrapterminator}{\relax}

\newenvironment{nuweb}[8]
    {\renewcommand{\nuwebscrapbottomcommand}{#2}%
    \renewcommand{\nuwebscraptitle}{#3}%
    \renewcommand{\nuwebscrapnumber}{#4}%
    \renewcommand{\nuwebscrapdeflist}{#5}%
    \renewcommand{\nuwebscrapdefs}{#6}%
    \renewcommand{\nuwebscrapreflist}{#7}%
    \renewcommand{\nuwebscraprefs}{#8}%
    #1%
    \begin{nuwebscrapcontents}}
    {\end{nuwebscrapcontents}%
    \nuwebscrapbottomcommand%
    \nuwebscrapterminator}
@}

We can also see that \textit{top-cmd} and \textit{bottom-cmd} have available
to them the six macros which are assigned the remaining six arguments, so
that the information about the scrap may be laid out in a variety of ways,
according to the definition of the commands. Finally, observe the
|\nuwebscrapterminator| command, which, although by default null, can be
defined to give a diamond at the end of every scrap, as in earlier versions
of nuweb.

We now define the \texttt{nuwebscrapcontents} environment: it is simply the
\texttt{alltt} environment, but with small text, indented by 0.5cm. Note that
in this environment, only braces and the backslash are special characters.
Nuweb assumes this to be the case, and only escapes those characters out when
formatting scraps; the environment \texttt{nuwebscrapcontents} must only be
redefined to an environment with similar properties.

@o nuweb.sty
@{
\newenvironment{nuwebscrapcontents}
    {\begin{list}{}%
    {\setlength{\leftmargin}{0.5cm}\setlength{\labelwidth}{0pt}%
    \setlength{\labelsep}{0pt}}%
    \item[]%
    \begin{small}\begin{alltt}}
    {\end{alltt}\end{small}\end{list}}
@}

There are two types of scrap: macro definitions and program file fragments. We
define two variants of the \texttt{nuweb} environment for each; these are
the environments placed in the |.tex| file by nuweb. For each type of
fragment there is an environment which allows page-breaks (for \texttt{@@D}
and \texttt{@@O}) and one which does not (for \texttt{@@d} and \texttt{@@o}).

@o nuweb.sty
@{
\newenvironment{nuwebbigmacro}[6]
    {\begin{flushleft}%
    \begin{nuweb}{\nuwebmacrotop}{\nuwebmacrobottom}{#1}{#2}{#3}{#4}{#5}{#6}}
    {\end{nuweb}%
    \end{flushleft}}
\newenvironment{nuwebmacro}[6]
    {\begin{flushleft}%
    \begin{minipage}[t]{\textwidth}%
    \begin{nuweb}{\nuwebmacrotop}{\nuwebmacrobottom}{#1}{#2}{#3}{#4}{#5}{#6}}
    {\end{nuweb}%
    \end{minipage}%
    \end{flushleft}}

\newenvironment{nuwebbigfile}[6]
    {\begin{flushleft}%
    \begin{nuweb}{\nuwebfiletop}{\nuwebfilebottom}{#1}{#2}{#3}{#4}{#5}{#6}}
    {\end{nuweb}%
    \end{flushleft}}
\newenvironment{nuwebfile}[6]
    {\begin{flushleft}%
    \begin{minipage}[t]{\textwidth}%
    \begin{nuweb}{\nuwebfiletop}{\nuwebfilebottom}{#1}{#2}{#3}{#4}{#5}{#6}}
    {\end{nuweb}%
    \end{minipage}%
    \end{flushleft}}
@}

We must now define the top and bottom commands:

@o nuweb.sty
@{
\newcommand{\nuwebmacrotop}%
    {\nuwebmacroname{\nuwebscraptitle}{\nuwebscrapnumber} \(\equiv\)%
    \vspace{-1ex}}
\newcommand{\nuwebmacrobottom}{%
    \vspace{-1ex}%
    \textrm{\footnotesize%
    \ifthenelse{\nuwebscrapdefs>1}{Macro defined by scraps
    \nuwebscrapdeflist.\\[-0.2\baselineskip]}{}%
    Macro %
    \ifthenelse{\nuwebscraprefs=0}{\textbf{not referenced}.}%
    {referenced in scrap}%
    \ifthenelse{\nuwebscraprefs>1}{s}{}%
    \ifthenelse{\nuwebscraprefs>0}{\ \nuwebscrapreflist.}{}%
    }}
\newcommand{\nuwebfiletop}{\textrm{\texttt{"\nuwebscraptitle"}
    \nuwebscrapnumber\ \(\equiv\)}%
    \vspace{-1ex}}
\newcommand{\nuwebfilebottom}{%
    \vspace{-1ex}%
    \textrm{\footnotesize%
    \ifthenelse{\nuwebscrapdefs=0}{}%
    {File defined by scrap%
    \ifthenelse{\nuwebscrapdefs>1}{s}{}%
    \ifthenelse{\nuwebscrapdefs>0}{\ \nuwebscrapdeflist.}{}%
    }%
    }}
@}

The next command formats a macro name:

@o nuweb.sty
@{
\newcommand{\nuwebmacroname}[2]%
    {\textrm{\(\langle\)#1\ #2\(\rangle\)}}
@}

Finally, we define two commands to control the appearance of the terminator
at the end of each scrap:

@o nuweb.sty
@{
\newcommand{\nuwebterminator}[1]{\renewcommand{\nuwebscrapterminator}{#1}}
\newcommand{\nuwebnoterminator}{\renewcommand{\nuwebscrapterminator}{\relax}}
@}

The package also contains the |nuwebindex| environment:

@o nuweb.sty
@{
\newcounter{nuwebindexcols}
\newenvironment{nuwebindex}[2][2]
    {\setcounter{nuwebindexcols}{#1}
     \columnseprule 0pt
     \columnsep 35pt
     \ifthenelse{\value{nuwebindexcols}>1}{\begin{multicols}%
         {\thenuwebindexcols}[#2]}{}
     \parindent 0pt
     \parskip 0pt plus .3pt\relax
     \raggedright
     \renewcommand{\item}{\par\filbreak\hangindent 40pt}}
     {\filbreak\ifthenelse{\value{nuwebindexcols}>1}{\end{multicols}}{}}
@}

Note the use of |\filbreak| to make sure that index entries are not broken
across a column or page break. The code above is based on the code for the
|theindex| environment.



\chapter{The documentation}

The user documentation is produced by the file |nuwebdoc.tex|, which simply
defines the token |\hidecode| before |\input|ting |nuweb.tex|. This together
with the |\if| clause in |nuweb.tex| causes only the first chapter and the
bibliography to be processed, omitting the code.

@o nuwebdoc.tex
@{
% Generate only the user documentation
\def\hidecode{true}
\input nuweb
@}

We must use only \TeX\ commands, as otherwise \LaTeX\ will complain that
there is a missing |\begin{document}|.

There is also a short \LaTeX\ file for users of previous versions of nuweb.

@O upgrade.tex
@{
% Upgrade notes

\documentclass{article}
\usepackage{shortvrb}
\MakeShortVerb{\|}

\begin{document}

\title{Changes to Nuweb since v0.87b}
\author{Reuben Thomas}
\date{25th February 1997}
\maketitle

\noindent The main changes as of v0.90 are as follows:

\begin{itemize}
\item The code has been changed from K\&R C to ANSI C.
\item The output is now for \LaTeXe\ rather than \LaTeX2.09.
\item All the typesetting support has been moved into a \LaTeX\ package, for
easy modification by the user.
\item The terminating symbol at the end of each scrap is now user-definable
(and optional).
\item Indices are more flexible; the user can choose in which index
environment to typeset them, and an environment is provided which allows
multiple-column typesetting without automatically starting a new chapter as
the default |theindex| environment does.
\item The functionality of both versions 0.87b and 0.87c has been retained,
so that \textsc{HTML} can be produced from \LaTeX\ or \textsc{HTML} input.
\item The formatting of scraps has been improved slightly to make source code
within webs easier to read during editing (the |-s| option).
\item |#line| directives can be ignored when testing to see if program
files need updating (the |-C| option).
\item A bug has been removed (I could only find one; Preston Briggs wrote
solid code!).
\item The documentation file may be named on the command line with the
|-o| flag; the original meaning of |-o| (do not generate program
files) has been changed to |-p|.
\end{itemize}

\end{document}
@}



\chapter{Indices} \label{indices}

Three sets of indices can be created automatically: an index of file names,
an index of macro names, and an index of user-specified identifiers. An index
entry includes the name of the entry, where it was defined, and where it was
referenced.


\begin{nuwebindex}{\section{Files}}
@f
\end{nuwebindex}


\begin{nuwebindex}[1]{\section{Macros}}
@m
\end{nuwebindex}


\begin{nuwebindex}{\section{Identifiers}}
@u
\end{nuwebindex}

\end{code}



\bibliographystyle{plain}
\bibliography{literate}


\end{document}